#!/usr/bin/perl -w

# Copyright 2013, 2014, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use FindBin;
use List::Util 'max','min';
use Math::Libm 'hypot';
use Math::Matrix;
use Math::PlanePath::PythagoreanTree;

use FindBin;
use lib::abs "$FindBin::Bin/../tools";
use MyTriples ':all';

# uncomment this to run the ### lines
use Smart::Comments;

{
  # parallelogram tiling tikz picture
  #
  #     000 000
  #     001 001
  #     002 002
  #     010 012  \
  #     011 011  |
  #     012 010  /
  #     020 020
  #     021 021
  #     022 022
  #     100 122  \
  #     101 121  |
  #     102 120  /    x,y
  #     110 110
  #     111 111
  #     112 112
  # x,y 120 102  \
  #     121 101  |
  #     122 100  /
  #
  my $x_max = 15;
  my $y_max = 8;

  my $path = Math::PlanePath::PythagoreanTree->new(coordinates => 'PQ',
                                                   digit_order => 'LtoH');

  my $draw_line = sub {
    my ($x1,$y1, $x2,$y2) = @_;
    ($x1,$y1, $x2,$y2) = line_clip($x1,$y1, $x2,$y2) or return;
    foreach my $v ($x1,$y1, $x2,$y2) {
      if ($v != int($v)) {
        $v = sprintf '%.3f', $v;  # mutate variables
      }
    }
    print "  \\draw ($x1, $y1) -- ($x2, $y2);\n";
  };

  my $draw_poly = sub {
    my @coords = @_;
    convex_poly_clip(\@coords);
    if (@coords) {
      print "  \\filldraw ";
      while (@coords) {
        my $x = shift @coords;
        my $y = shift @coords;
        print "($x, $y)--";
      }
      print "cycle;\n";
    }
  };

  my $max_depth = 0;
  foreach my $x (0 .. $x_max) {
    foreach my $y (0 .. $y_max) {
      my $n = $path->xy_to_n($x,$y) // next;
      {
        my ($n_u, $n_a, $n_d) = $path->tree_n_children($n);
        my ($x_u,$y_u) = $path->n_to_xy($n_u);
        my ($x_a,$y_a) = $path->n_to_xy($n_a);
        my ($x_d,$y_d) = $path->n_to_xy($n_d);

        $draw_poly->($x,$y,
                     $x_u,$y_u,
                     $x_a,$y_a,
                     $x_d,$y_d);
      }
      my $depth = $path->tree_n_to_depth($n);
      my $n_start = $path->tree_depth_to_n($depth);
      my $offset = $n - $n_start;
      $max_depth = max($max_depth, $depth);
      # {
      #   my $n_next = ternary_1s_reverse(ternary_1s_unreverse($n) + 1);
      #   my $depth_next = $path->tree_n_to_depth($n_next);
      #   if ($depth_next == $depth) {
      #     my ($x_next,$y_next) = $path->n_to_xy($n_next);
      #     $draw_line->($x,$y, $x_next,$y_next);
      #   }
      # }
      # {
      #   my $n_prev = ternary_1s_reverse(ternary_1s_unreverse($n) - 1);
      #   my $depth_prev = $path->tree_n_to_depth($n_prev);
      #   if (defined $depth_prev && $depth_prev == $depth) {
      #
      #     my ($x_prev,$y_prev) = $path->n_to_xy($n_prev);
      #     if ($x_prev > $x_max || $y_prev > $y_max) {
      #
      #       $draw_line->($x,$y, $x_prev,$y_prev);
      #     }
      #   }
      # }
    }
  }

  # print "  % max_depth $max_depth\n";
  foreach my $depth (0 .. $max_depth) {
    my ($prev_x,$prev_y);
    foreach my $n ($path->tree_depth_to_n($depth) .. $path->tree_depth_to_n_end($depth)) {
      my $n_depth = $path->tree_depth_to_n($depth);
      my $offset = $n - $n_depth;

      $offset = ternary_1s_reverse($offset);
      $n = $n_depth + $offset;

      my ($x,$y) = $path->n_to_xy($n);

      if (defined $prev_x) {
        $draw_line->($prev_x,$prev_y, $x,$y);
      }
      $prev_x = $x;
      $prev_y = $y;
    }
  }

  # return new ($x1,$y1, $x2,$y2) which is clipped to $x_max,$y_max
  # return empty list if all outside
  sub line_clip {
    my ($x1,$y1, $x2,$y2) = @_;
    return if $x1 > $x_max && $x2 > $x_max;
    return if $y1 > $y_max && $y2 > $y_max;

    my ($x1new,$y1new) = line_end_clip($x2,$y2, $x1,$y1, $x_max,$y_max);
    ($x2,$y2) = line_end_clip($x1,$y1, $x2,$y2);
    return ($x1new,$y1new, $x2,$y2);
  }

  sub convex_poly_clip {
    my ($aref) = @_;
    ### convex_poly_clip(): $aref

    foreach my $side (1 .. 4) {  # each side
      ### side: $_
      my $max = ($side & 1 ? $y_max : $x_max);

      for (my $i = 0; $i < $#$aref && $#$aref >= 3; ) {
        ### at: "i=$i of ".scalar(@$aref)."  ".join(', ',@$aref)
        my $y = $aref->[$i+1];
        if ($y <= $max) {
          # This vertex is below the $max limit, keep it unchanged.
          $i += 2;

        } else {
          # This vertex is outside the $max limit, replace it by zero, one or
          # two new clipped points.
          my ($x,$y) = splice @$aref, $i,2;

          {
            my $yprev = $aref->[$i-1];  # with possible wrap back to $xn,$yn
            if ($yprev <= $max) {
              my $xprev = $aref->[$i-2];
              my $xnew = ($x*($max - $yprev) + $xprev*($y - $max)) / ($y-$yprev);
              splice @$aref, $i,0, $xnew,$max;
              $i += 2;
            } else {
              # $yprev and $y both above $max limit, so nothing for segment
              # $yprev to $y, just leave $yprev for the next vertex to
              # consider.  (This case only occurs when $i==0 and so $yprev is
              # wrapped back to the last vertex $yn.  Any later $i will have
              # $yprev already clipped to $yprev<=$max.)
            }
          }

          {
            my $inext = $i % scalar(@$aref);
            my $ynext = $aref->[$inext+1];
            if ($ynext <= $max) {
              my $xnext = $aref->[$inext];
              my $xnew = ($x*($max - $ynext) + $xnext*($y - $max)) / ($y-$ynext);
              splice @$aref, $i,0, $xnew,$max;
              $i += 2;
            } else {
              # $y and $ynext both above $max limit, so nothing for segment $y
              # to $ynext
            }
          }
        }
      }

      # rotate 90
      for (my $i = 0; $i < $#$aref; $i += 2) {
        ($aref->[$i],$aref->[$i+1]) = ($aref->[$i+1], -1 - $aref->[$i]);
      }
    }
    if (@$aref == 2) {
      @$aref = ();
    }
  }

  exit 0;
}
{
  # LPT matrix
  my $lpt = Math::Matrix->new([.5, 1, -.5],
                              [1, -1, 1],
                              [.5, -1, 1.5]);
  print "lpt\n",$lpt;
  print $lpt * Math::Matrix->new([3],[4],[5]);
  print $lpt * Math::Matrix->new([5],[12],[13]);
  print "inverse\n",$lpt->invert;
  print "squared\n",$lpt*$lpt;

  # p = d + q
  my $dq_to_pq = Math::Matrix->new([1,1],[0,1]);
  my $pq_to_dq = $dq_to_pq->invert;
  print "dq_to_pq\n",$dq_to_pq;
  print "pq_to_dq\n",$pq_to_dq;
  my $swap = Math::Matrix->new([0,1],[1,0]);
  print "swap\n",$swap;
  my $conj_on_dq = $swap;
  my $conj_on_pq = $dq_to_pq * $conj_on_dq * $pq_to_dq;
  print "conj_on_pq\n",$conj_on_pq;
  my $conj_on_pq_3x3 = mm_2x2_to_3x3($conj_on_pq);
  print "conj_on_pq_3x3\n",$conj_on_pq_3x3;
  exit 0;

  print <<HERE;
  dq_to_pq = [1,1; 0,1]
  pq_to_dq = dq_to_pq^-1
  pq_to_dq * [a,b; c,d] * dq_to_pq
    dq_to_pq * [0,1; 1,0] * pq_to_dq
  m = dq_to_pq * [e,f; g,h] * pq_to_dq
    /* = [e+g, -e+f-g+h;  g,     -g+h] */
  abcd_to_3x3(a,b,c,d) = [ ((a*a - c*c) - (b*b - d*d)) / 2,  \
                           a*b - c*d,                        \
                           ((a*a - c*c) + (b*b - d*d)) / 2;  \
                                                             \
                           a*c - b*d,                        \
                           a*d + b*c,                        \
                           a*c + b*d;                        \
                                                             \
                           ((a*a + c*c) - (b*b + d*d)) / 2,  \
                           a*b + c*d,                        \
                           ((a*a + c*c) + (b*b + d*d)) / 2 ]
  m3x3_to_2x2(m) =  abcd_to_3x3(m[1,1],m[1,2],m[2,1],m[2,2])
  m3x3_to_2x2(m)
HERE

# [(f + h)*e + (-1/2*f^2 + (g - h)*f)
#  -e^2 + (f + (-2*g + h))*e + g*f
#  e^2 + (-f + (2*g - h))*e + (1/2*f^2 + (-g + h)*f)]
# 
# [h*e + ((g - h)*f + (2*h*g - h^2))
#  (-2*g + h)*e + (g*f + (-2*g^2 + 2*h*g))
#  (2*g - h)*e + ((-g + h)*f + (2*g^2 - 2*h*g + h^2))]
# 
# [(f + h)*e + (-1/2*f^2 + (g - h)*f + (2*h*g - h^2))
#  -e^2 + (f + (-2*g + h))*e + (g*f + (-2*g^2 + 2*h*g))
#  e^2 + (-f + (2*g - h))*e + (1/2*f^2 + (-g + h)*f + (2*g^2 - 2*h*g + h^2))]

# [ ef + eh + fg - fh + -1/2*f^2
#   + ef - 2eg + eh + fg -e^2
#   + -ef + 2eg - eh + -fg + fh + e^2 + 1/2*f^2
# 
# [ eh + fg - fh + 2gh - h^2
#   -2eg + eh + fg + 2gh + -2*g^2
#   2eg - eh + -fg + fh - 2gh + h^2 + 2*g^2
# 
# [(f + h)*e + (-1/2*f^2 + (g - h)*f + (2*h*g - h^2))
#  -e^2 + (f + (-2*g + h))*e + (g*f + (-2*g^2 + 2*h*g))
#  e^2 + (-f + (2*g - h))*e + (1/2*f^2 + (-g + h)*f + (2*g^2 - 2*h*g + h^2))]


}

{
  # LPT matrix vs 2x2
  my $limit = 5;
  foreach my $a (-$limit .. $limit) {
    foreach my $b (-$limit .. $limit) {
      foreach my $c (-$limit .. $a-1) {
        foreach my $d (-$limit .. $limit) {
          my $m = Math::Matrix->new([$a/2,$b/2],[$c/2,$d/2]);
          my $m3 = mm_2x2_to_3x3($m);
          if ($m3->[0][0] == .5
              && $m3->[0][1] == 1
              && $m3->[0][2] == -.5

              # && $m3->[1][0] == 1
              # && $m3->[1][1] == -1
              # && $m3->[1][2] == 1

              # && $m3->[2][0] == .5
              # && $m3->[2][1] == -1
              # && $m3->[3][2] == 1.5
             ) {
            print "[$a,$b; $c,$d]\n$m3";
          }
        }
      }
    }
  }

  exit 0;
}
{
  # row area

  #   2,1 3,2 5,2 4,1                area = 1+2*1/2 = 2
  #
  # 2|         1     .     2
  #  |
  # 1|    0    .     3
  #   ------------------------
  #       2    3     4     5


  #    0   1   2   3   4    5   6   7   8   9       area = 14
  #   2,1 4,3 8,5 7,4 9,4 12,5 8,3 7,2 9,2 6,1
  #
  # 5|                                  2                   5
  #  |
  # 4|                             3         4
  #  |
  # 3|              1                   6
  #  |
  # 2|                             7         8
  #  |
  # 1|    0                   9
  #   ---------------------------------------------------------
  #       2    3    4    5    6    7    8    9   10   11   12
  #
  #  0,7,8,9 area = 7 - 5/2 - 3/2 = 3

  #               0   9   8    5   2   1
  # Convex hull  2,1 6,1 9,2 12,5 8,5 4,3       area = 21
  #
  # 4/2 + 4*2/2 + 3*3/2 + 3/2 + 3 + 4 + 2 = 21
  #
  # 5|                                  2------------------ 5
  #  |                                  |    |
  # 4|                     4*2/2        | 3  |   3*3/2
  #  |                                  |    |
  # 3|              1-------------------|    |
  #  |              |      4            |    |
  # 2|         4/2  |------------------------8
  #  |              |    2    |    3/2
  # 1|    0-------------------9
  #   ---------------------------------------------------------
  #       2    3    4    5    6    7    8    9   10   11   12


  require Math::Geometry::Planar;
  require Math::ConvexHull;
  my $path = Math::PlanePath::PythagoreanTree->new(coordinates => 'PQ',
                                                   digit_order => 'LtoH');
  my @values;
  foreach my $depth (1 .. 7) {
    my @points = ([2,1]);
    my $n_depth = $path->tree_depth_to_n($depth);
    foreach my $offset (0 .. 3**$depth-1) {
      my $n = $n_depth + ternary_1s_reverse($offset);
      my ($x,$y) = $path->n_to_xy($n);
      push @points, [$x,$y];
    }

    my $points = \@points;
    my $planar = Math::Geometry::Planar->new;
    $planar->points($points);
    my $area = $planar->area;
    print "$depth  $area\n";
    if (@points <= 10) {
      print "   ",points_str($points),"\n";
    }

    # $points = Math::ConvexHull::convex_hull($points);
    # my $num_points = scalar(@points);
    # $planar->points($points);
    # $area = $planar->area;
    # print "    points=$num_points $area\n";
    # if (@points <= 10) {
    #   print "    ",points_str($points),"\n";
    # }
    push @values, $area;
  }

  shift @values;

  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values);

  @values = first_differences(@values);
  Math::OEIS::Grep->search(array => \@values,
                           name => 'differences');
  exit 0;

  sub points_str {
    my ($points) = @_;
    return  join(' ',map{join(',',@$_)}@$points)
  }
  sub first_differences {
    my $prev = shift;
    return map { my $diff = $_-$prev; $prev = $_; $diff } @_;
  }
}
{
  # UArD row area
  my $path = Math::PlanePath::PythagoreanTree->new(tree_type => 'UArD',
                                                   coordinates => 'PQ');
  my $scale = 2;
  require Image::Base::GD;
  my $width = 1300;
  my $height = 900;
  my $image = Image::Base::GD->new(-width => $width, -height => $height);
  $image->rectangle (0,0,$width-1,$height-1,'black');
  my @values;
  foreach my $depth (8) {
    my $n_row = $path->tree_depth_to_n($depth);
    my $n_end = $path->tree_depth_to_n_end($depth);
    foreach my $n ($n_row .. $n_end-1) {
      my ($x1,$y1) = $path->n_to_xy($n);
      my ($x2,$y2) = $path->n_to_xy($n+1);
      $x1 -= $y1;
      $x2 -= $y2;
      $x1 *= $scale;
      $y1 *= $scale;
      $x2 *= $scale;
      $y2 *= $scale;
      $image->line($x1,$y1, $x2,$y2, 'white');
    }
    $image->save('/tmp/x.png');
    require IPC::Run;
    IPC::Run::run (['xzgv','/tmp/x.png']);
    # IPC::Run::run (['see','--debug','/tmp/x.png']);
    # IPC::Run::run (['conkeror','/tmp/x.png']);
  }
  exit 0;
}

{
  # UMT iteration
  # T = (1 3)
  #     (0 2)
  # while p >= 2q and p odd q even
  # T^2 = (1  3+3*2 )
  #       (0  2*2   )
  # T^n = (1  3*(2^n-1)
  #     = (0  2^n)
  # T^-1 = (1 -3/2)
  #        (0 1/2)
  # m = trailing 0 bits of q except stop if p < 2q from a U leg
  # p -= 3*q/2
  # p -= 3*(q/2 + q/4)
  # p - 3*q*(1/2 + 1/4) < 2q
  # p - 3*q*(1 - 1/2^m) < 2q
  # p < 2q + 3*q*(1 - 1/2^m)
  # 3*q*(1 - 1/2^m) > p - 2q
  # (1 - 1/2^m) > (p - 2q)/3q
  # 1 - 1/2^m > (p/q - 2)/3
  # - 1/2^m > (p/q - 2)/3 - 1
  # 1/2^m < 1 - (p/q - 2)/3
  # 1/2^m < 1 - (p - 2q)/3q
  # 1/2^m < (3q - p + 2q)/3q
  # 1/2^m < (5q - p)/3q
  # 3q/(5q - p) < 2^m
  #
  my $path = Math::PlanePath::PythagoreanTree->new(coordinates => 'PQ',
                                                   tree_type => 'UMT');
  my $p = 2;
  my $q = 1;
  foreach my $n ($path->n_start .. 60) {
    my ($x,$y) = $path->n_to_xy($n);
    print "$p,$q  $x,$y\n";
    ($p,$q) = umt_pq_next($p,$q);
  }
  exit 0;

  sub umt_pq_next {
    my ($p,$q) = @_;

    {
      my $divisor = 2*$q;
      my $m = int(($p-$q) / $divisor);
      my $r = $p - $divisor*$m;
      ### umt_pq_next(): "$p,$q  m=$m r=$r"

      $p == $m*$divisor + $r or die;
      $r > $q or die;
      $r < $q+2*$q or die;

      if ($r == 2*$q) {
        ### U^(m+1) ...
        # $m++;
        # $r -= $divisor;
        # $m--;
        # $r += $divisor;
        # return (($m+2) * $r + -($m+3)*$q,
        #         ($m+1) * $r + -($m+2)*$q);

        # ($m+2) * $r + -($m+1)*$q      U^(m+1)
        # ($m+1) * $r + -($m)  *$q
        return ($m+3, $m+2);

      } elsif ($r < 2*$q) {
        ### reverse U ...
        return (-($m+1) * $r + (3*$m+4)*$q,
                -$m     * $r + (3*$m+1)*$q);
      } else {
        ### reverse A ...
        return (($m+2) * $r + -($m+3)*$q,
                ($m+1) * $r + -($m+2)*$q);
      }
    }
  }
}

{
  # row area convex hull

  require Math::Geometry::Planar;
  require Math::BaseCnv;
  my $path = Math::PlanePath::PythagoreanTree->new(coordinates => 'PQ',
                                                   tree_type => 'UArD',
                                                   digit_order => 'LtoH');
  my @values;
  my $prev_log = 0;
  foreach my $depth (2 .. 9) {
    print "depth=$depth\n";
    my $points = [ [2,1] ];
    my $n_row = $path->tree_depth_to_n($depth);
    foreach my $n ($path->tree_depth_to_n($depth) ..
                   $path->tree_depth_to_n_end($depth)) {
      my ($x,$y) = $path->n_to_xy($n);
      push @$points, [$x,$y];
    }

    my $planar = Math::Geometry::Planar->new;
    $planar->points($points);
    my $value;

    # {
    #   # bbox, max(X) = Pell 12,29,70
    #   #       max(Y) = Pell 5,12,29
    #   #       area = pell*pell = 297,1904,11523,68208,400081
    #   $planar = $planar->bbox;
    #   $points = $planar->points;
    #   # $value = $points->[0]->[0];
    #    $value = $planar->area;
    # }

    {
      # convex hull
      # num points = 2n = 8,10,12,14,16
      # area = 0,2,139,834,4892,28551,166453
      #   dlog=1.762758
      # perimeter = 60.88, 152.99, 375.30, 912.00, 2207.67, 5335.71, 12887.44
      #   dlog=0.881831 exp=2.4153
      $planar = $planar->convexhull;
      $points = $planar->points;
      $value = $planar->area;
      # $value = $planar->perimeter;
      # $value = scalar(@$points);

      foreach my $p (reverse @$points) {
        my $n = $path->xy_to_n(@$p);
        if ($n >= $n_row) { $n -= $n_row; }
        my $n3 = Math::BaseCnv::cnv($n,10,3);
        $n3 = sprintf '%0*s', $depth, $n3;
        printf "  %4d,%4d  %8s  [%s]\n",
          $p->[0],$p->[1], $n,$n3;
      }
    }

    # {
    #   # perimeter = 140.57, 774.39, 4451.85, 25868.97, 150680.50, 878118.84
    #   #   dlog=1.762722
    #    $value = $planar->perimeter * 139 / 140.57;
    #   # $value = $planar->centroid()->[0];
    # }

    push @values, $value;
    my $num_points = scalar(@$points);
    my $log = log($value);
    my $dlog = $log - $prev_log;
    printf "    points=%d value=$value  log=%.4f dlog=%.6f\n",
      $num_points, $log, $dlog;
    if (@$points <= 10) {
      print "   ",points_str($points),"\n";
    }
    $prev_log = $log;
  }

  shift @values;

  print join(',',map {sprintf '%.2f', $_} @values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values);
  exit 0;
}

{
  # Dickson r,s,t
  my $path_pq = Math::PlanePath::PythagoreanTree->new(coordinates => 'PQ');
  my $path_ab = Math::PlanePath::PythagoreanTree->new(coordinates => 'AB');
  my $path_ac = Math::PlanePath::PythagoreanTree->new(coordinates => 'AC');
  foreach my $i ($path_pq->n_start .. 20) {
    my ($p,$q) = $path_pq->n_to_xy($i);
    my ($a,$b) = $path_ab->n_to_xy($i);
    (undef, my $c) = $path_ac->n_to_xy($i);
    my $r = $b+$c;
    my $s = $c-$a;
    my $t = $a+$b-$c;

    print "$i  $a,$b,$c [$p,$q]  $r,$s,$t\n";
  }
  exit 0;
}

{
  # Tong
  my $path_pq = Math::PlanePath::PythagoreanTree->new(coordinates => 'PQ');
  my $path_ab = Math::PlanePath::PythagoreanTree->new(coordinates => 'AB');
  my $path_ac = Math::PlanePath::PythagoreanTree->new(coordinates => 'AC');
  foreach my $i ($path_pq->n_start .. 20) {
    my ($m,$n) = $path_pq->n_to_xy($i);
    my ($a,$b) = $path_ab->n_to_xy($i);
    (undef, my $c) = $path_ac->n_to_xy($i);
    my $p = $n;
    my $q = $m-$n;
    # my $a = ($as + 2*$bs - $cs)/2;
    # my $b = ($as - $bs + $cs);
    # my $c = ($as - 2*$bs + 3*$cs)/2;

    my $as = ($a + 2*$b - $c)/2;
    my $bs = ($a - $b + $c);
    my $cs = ($a - 2*$b + 3*$c)/2;

    print "$i  $a,$b,$c [$m,$n]  $as,$bs,$cs\n";
  }
  exit 0;
}

{
  # p' = a.-b+b.1    p=abs(b) q=1
  # q' = c.-b

  # SB filtered as UAD LtoH cf Katayama
  #
  # r*(l)*r^-1 == u
  # r*(r^2)*r^-1 == d
  # r*(r*l*s)*r^-1 == a
  # r*(s*l*r)*r^-1 == a
  #
  # R*tree = 2,1 etc
  # U * R*tree = 3,2 etc on each not-both-odd point
  # R*L*R^-1 * R*tree = R*L*tree
  #
  # U map 2,1 -> 3,2
  # R*L*tree = U * R*tree
  # R*L = U * R
  # U * R = R*L
  # U = R*L*R^-1
  #
  # A * R*tree = 5,2 etc swapped
  # R*(R*L*S)*R^-1 * R*tree
  # = R*(R*L*S) * tree
  #
  # A * R*tree = R*R*L*S*tree
  #
  # R*R*R*tree = D * R*tree
  # D = R*R
  #
  my $depth = 5;

  require Math::PlanePath::RationalsTree;
  my $path_sb = Math::PlanePath::RationalsTree->new(tree_type => 'SB');

  my $print_tree;
  $print_tree = sub {
    my ($tree, $indent) = @_;
    $indent ||= 0;
    print((' 'x$indent),$tree->{'p'},',',$tree->{'q'},"\n");
    $indent += 2;
    foreach my $child (@{$tree->{'children'}}) {
      $print_tree->($child, $indent)
    }
  };

  my $make_tree;
  $make_tree = sub {
    my ($path, $depth, $p,$q) = @_;
    ### make_tree: "$p,$q"
    my $ret = { p => $p, q => $q, children => [] };
    if (--$depth >= 0) {
      my $n = $path->xy_to_n($p,$q) or die;
      foreach my $n_child ($path->tree_n_children($n)) {
        my ($p_child,$q_child) = $path->n_to_xy($n_child) or die;
        push @{$ret->{'children'}}, $make_tree->($path, $depth, $p_child,$q_child);
      }
    }
    return $ret;
  };

  my $sb_tree = $make_tree->($path_sb, $depth, 2,1);
  ### $sb_tree
  $print_tree->($sb_tree);

  my $filter_tree;
  $filter_tree = sub {
    my ($tree) = @_;
    my @new_children;
    my @pending = @{$tree->{'children'}};
    while (@pending) {
      my $child = shift @pending;
      if (pq_acceptable($child->{'p'},$child->{'q'})) {
        push @new_children, $child;
      } else {
        unshift @pending, @{$child->{'children'}};
      }
    }
    $tree->{'children'} = \@new_children;
    foreach my $child (@new_children) {
      $filter_tree->($child);
    }
  };

  $filter_tree->($sb_tree);
  ### $sb_tree
  $print_tree->($sb_tree);

  my $prune_to_3;
  $prune_to_3 = sub {
    my ($tree) = @_;
    if (scalar(@{$tree->{'children'}}) < 3) {
      $tree->{'children'} = [];
    } else {
      foreach my $child (@{$tree->{'children'}}) {
        $prune_to_3->($child);
      }
    }
  };

  $prune_to_3->($sb_tree);
  print "SB pruned\n";
  $print_tree->($sb_tree);

  my $path_uad = Math::PlanePath::PythagoreanTree->new(tree_type => 'UAD',
                                                       coordinates => 'PQ',
                                                       digit_order => 'LtoH');
  print "UAD\n";
  $print_tree->($make_tree->($path_uad, 3, 2,1));

  #   return { p => 2,
  # my $tree =
  #
  # my $sb_filtered_n_is_good = sub {
  #   my ($n) = @_;
  #   my ($x,$y) = $path->n_to_xy($n) or return 0;
  #   return pq_acceptable($x,$y);
  # };
  # my $filtered_n_children = sub {
  #   my ($n) = @_;
  #   my ($x,$y) = $path->n_to_xy($n);
  #   unless ($filtered_xy_is_good->($x,$y)) {
  #     die "oops, n=$n xy=$x,$y both odd";
  #   }
  #   my @ret;
  #   foreach my $n_child ($path->tree_n_children($n)) {
  #     my ($x,$y) = $path->n_to_xy($n_child);
  #     if ($filtered_n_is_good->($n_child)) {
  #       push @ret, $n_child;
  #     } else {
  #       push @ret, $path->tree_n_children($n_child);
  #     }
  #   }
  #   return @ret;
  # };
  #
  # my $count = 0;
  # my @pending = ($path->xy_to_n(2,1));
  # foreach (1 .. 2) {
  #   my @new_pending;
  #   while (@pending) {
  #     my $n = shift @pending;
  #     $count++;
  #     my ($x,$y) = $path->n_to_xy($n);
  #     my @n_children = $filtered_n_children->($n);
  #     my @xy_children = $filtered_xy_children->($x,$y);
  #     print "$n  $x,$y  children ",scalar(@n_children),"\n";
  #     foreach my $n_child (@n_children) {
  #       push @new_pending, $n_child;
  #       my ($x_child,$y_child) = $path->n_to_xy($n_child);
  #       print "   child $n_child  $x_child,$y_child matrix $xy_children[0],$xy_children[1]\n";
  #       unless ($x_child == $xy_children[0]
  #               && $y_child == $xy_children[1]) {
  #         warn "diff $x_child,$y_child matrix $xy_children[0],$xy_children[1]";
  #       }
  #       shift @xy_children;
  #       shift @xy_children;
  #     }
  #   }
  #   @pending = @new_pending;
  # }
  # print "count $count\n";
  exit 0;
}


{
  # FB tree P,Q row totals

  print U_2x2()+A_2x2()+D_2x2();
  print M1_2x2()+M2_2x2()+M3_2x2();
  print U_2x2()+M2_2x2()+T_2x2();
  my $path = Math::PlanePath::PythagoreanTree->new (coordinates => 'PQ',
                                                    tree_type => 'FB');
  my @P_total;
  my @Q_total;
  foreach my $depth (0 .. 10) {
    my ($n_row, $n_end) = $path->tree_depth_to_n_range($depth);
    my $P_total = 0;
    my $Q_total = 0;
    foreach my $n ($n_row .. $n_end) {
      my ($p,$q) = $path->n_to_xy($n);
      $P_total += $p;
      $Q_total += $q;
    }
    push @P_total, $P_total;
    push @Q_total, $Q_total;
    my $P_formula = FB_P_formula($depth);
    my $P_recurrence = FB_P_recurrence($depth);
    my $Q_recurrence = FB_Q_recurrence($depth);
    my ($P_rec, $Q_rec) = FB_PQ_recurrence($depth);
    print "$depth   $P_total,$Q_total   $P_rec,$Q_rec  $P_recurrence,$Q_recurrence $P_formula\n";
  }
  print "P rows\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@P_total);
  print "\n";

  print "Q rows\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@Q_total);
  print "\n";
  exit 0;

  sub FB_P_recurrence {
    my ($k) = @_;
    if ($k < 0) { die; }
    if ($k == 0) { return 2; }
    if ($k == 1) { return 11; }
    return 7*FB_P_recurrence($k-1) - 8*FB_P_recurrence($k-2);
  }
  sub FB_Q_recurrence {
    my ($k) = @_;
    if ($k < 0) { die; }
    if ($k == 0) { return 1; }
    if ($k == 1) { return 6; }
    return 7*FB_Q_recurrence($k-1) - 8*FB_Q_recurrence($k-2);
  }
  sub FB_PQ_recurrence {
    my ($k) = @_;
    if ($k < 0) { die; }
    if ($k == 0) { return (2, 1); }
    my ($P,$Q) = FB_PQ_recurrence($k-1);
    return (5*$P+$Q, 2*$P+2*$Q);
  }
  sub FB_P_formula {
    my ($k) = @_;
    return 2*6**$k;
  }
}
{
  # table of CW p,q iterates
  #
  my $step = 0;
  my $p = 2;
  my $q = 1;
  my $count = 0;
  for (;;) {
    if (pq_acceptable($p,$q)) {
      last if $count++ > 6;
      if ($step == 0) {
        print "start       ";
      } else {
        print "step by $step to";
      }
      my ($A,$B,$C) = pq_to_abc($p,$q);
      printf "   & %d, & %d   & %2d & %2d & %2d \\\\\n",
        $p,$q, $A,$B,$C;
      $step = 0;
    }
    $step++;
    ($p,$q) = CW_pq_next($p,$q)
  }
  exit 0;
}
{
  # CW iterate p,q
  #

  my $L = Math::Matrix->new([1,0],[1,1]);
  my $R = Math::Matrix->new([1,1],[0,1]);
  my $Linv = $L->clone->invert;
  my $Rinv = $R->clone->invert;
  ### R   : math_matrix_string($R)
  ### Rinv: math_matrix_string($Rinv)

  #            0/1
  #        /         \
  #    0/1              1/1
  #     \              /
  #       1/1        1/2
  #     /    \        /  \
  #  1/2      2/1   1/3   3/2          p=2;q=1  m=2
  # two steps
  # Rinv^(m-1) to 1/1
  # Rinv^m to 0/1
  # Linv to 0/1
  # R to 1/1          r*[1;0] == [1;0]
  # L^(m-1) to 1/2
  # R to 3/2
  # up is 1 level short since Linv * 0/1 = 0/1
  #
  # q, p+q-2r
  # R*Linv = [0,1; -1,1]
  # l=[1,0;1,1]; r=[1,1;0,1]
  # R*(L^-1)*[q; p+q-2*r] = [p - 2*r) + q; p - 2*r]
  # so two p+2r+q, p-2r
  #
  my $two = sub {
    my ($p,$q) = @_;
    ### two: "$p, $q"
    die "two $p, $q" unless $p > 0 && $q > 0;
    {
      my $r = $p%$q;
      my $p2r = $p - 2*$r;
      return ($p2r+$q, $p2r);
    }
    my $r = $p%$q;
    my $m = ($p-$r)/$q;
    my $pq = Math::Matrix->new([$p],[$q]);
    ### two: mvec_to_str($pq)
    ### $m
    ### two up: mvec_to_str(matrix_pow($Rinv,$m) * $pq)

    my $next = ($R * matrix_pow($L,$m-1) * $R*$Linv * matrix_pow($Rinv,$m))
      * $pq;
    return mvec_to_pq($next);
  };

  # step 1  q, p+q-2r
  # R*R*L^-2*[q; p+q-2*r] = [-q + (2*p - 4*r)] [-q + (p - 2*r)]
  #
  # R*R*L^-1*R^-1*[p;q] = [3q-p; 2q-p]  eg. [8;5] -> [7;2]
  # then two [p - 2*r + q; p - 2*r]
  # r = 3q-p % (2q-p)
  # m = floor 3q-p / (2q-p)   >= 2
  # 3q-p = m*(2q-p) + r
  # m=1 so p%q = p-q
  #
  # up part L^-1*R^-1*[p;q] = [p-q; 2q-p]
  # two part
  # (3q-p)-2r+(2q-p); 3q-p -2r
  # = 5q-2p-2r; 3q-p-2r


  # R^2*L^-2*[q; p+q-2*r] = [p - 2*r) + q; p - 2*r]
  my $four = sub {
    my ($p,$q) = @_;
    {
      my $r = $p%$q;
      if ($p >= 2*$q) {
        { my $p2r = $p - 2*$r; return (2*$p2r - $q, $p2r - $q); }
        return (2*$p-4*$r - $q, $p-2*$r - $q);
      } else {
        { $r = ($p-$q) % (2*$q-$p);
          return (5*$q-2*$p-2*$r, 3*$q-$p-2*$r);
        }
        return $two->(3*$q-$p, 2*$q-$p);
        # ($p,$q) = (3*$q-$p, 2*$q-$p);
      }
    }
  };

  my $p = 2;
  my $q = 1;
  foreach (1 .. 50) {
    my ($four_p,$four_q) = $four->($p,$q);
    ($p,$q) = CW_pq_next($p,$q);
    ($p,$q) = CW_pq_next($p,$q);
    { my ($p,$q) = CW_pq_next($p,$q);
      ($p,$q) = CW_pq_next($p,$q);
      my $diff = ($p == $four_p && $q == $four_q ? '' : '****');
      print "$p $q  $four_p $four_q$diff\n";
    }
  }
  exit 0;

  sub matrix_pow {
    my ($m, $exp) = @_;
    if ($exp < 0) {
      $m = $m->clone->invert;
    }
    my $ret = Math::Matrix->new_identity($m->size);
    foreach (1 .. $exp) {
      $ret *= $m;
    }
    ### pow: math_matrix_string($ret)
    return $ret;
  }
  sub CW_pq_next {
    my ($p,$q) = @_;
    return ($q, $p+$q - 2*($p%$q));
  }
}
{
  # CW iterate A,B maybe
  #
  #           X                     Y
  #     N is ---      N+1 is ---------------
  #           Y              X+Y - 2*(X % Y)      0 <= X%Y < Y
  #
  # N+2 is      X+Y - 2*(X % Y)      0 <= X%Y < Y
  #             ---------------
  # Y+(X+Y - 2*(X % Y)) - 2*(Y % (X+Y - 2*(X % Y)))

  # A = X^2-Y^2        X = qY+r      r = X%Y q=floor(X/Y)
  # B = 2XY
  # C = sqrt(A^2+B^2) = X^2+Y^2
  # X = sqrt (C+A)/2
  # Y = sqrt (C-A)/2
  #
  # Anext = Y^2 - (X+Y - 2*(X % Y))^2
  #       = Y^2 - (X^2 + XY - 2X(X%Y)
  #               + YX + Y^2 + - 2Y(X%Y)
  #               -2X(X%Y)X + -2X(X%Y)Y  + 4(X%Y)^2)
  #       = Y^2 - (X^2 + 2XY + Y^2
  #                - 4X(X%Y) - 4Y(X%Y) + 4(X%Y)^2)
  #       = Y^2 - X^2 - 2XY - Y^2 + 4X(X%Y) + 4Y(X%Y) - 4(X%Y)^2
  #       = - X^2 - 2XY + 4X(X%Y) + 4Y(X%Y) - 4(X%Y)^2

  # sqrt((C+A)/2) = sqrt((C-A)/2)*q + r

  my $pq_next = sub {
    my ($p,$q) = @_;
    return ($q, $p+$q - 2*($p%$q));
  };
  my $a_next = sub {
    my ($p,$q) = @_;
    my $r = $p % $q;
    return 4*$r*($p+$q - $r) - ($p+2*$q)*$p;
    return 4*($p+$q)*$r - 4*$r**2 - ($p+2*$q)*$p;
    return - $p**2 - 2*$p*$q + 4*($p+$q)*$r - 4*$r**2;
    return $q*$q - ($p+$q)**2 + 4*($p+$q)*$r - 4*$r**2;
    return $q*$q - ($p+$q - 2*$r)**2;
  };
  my $p = 1;
  my $q = 1;
  foreach (1 .. 20) {
    my ($pnext,$qnext) = CW_pq_next($p,$q);
    if (pq_acceptable($pnext,$qnext)) {
      my $a = $pnext*$pnext - $qnext*$qnext;
      my $g = $a_next->($p,$q);
      print "$p $q  $a $g\n";
    }
    ($p,$q) = ($pnext,$qnext);
  }
  exit 0;
}


{
  # UArD horizontal and diagonal lengths PNG picture

  require Math::BigInt;
  Math::BigInt->import (try => 'GMP');
  require Set::IntSpan::Fast;
  my $x_limit = 100;
  my $y_limit = 20;
  my $scale = 20;
  my $depth_limit = int($scale * .75);

  my $path = Math::PlanePath::PythagoreanTree->new (tree_type => 'UArD',
                                                    coordinates => 'PQ');
  my $n_start = $path->n_start;

  my $width  = $x_limit * $scale;
  my $height = $y_limit * $scale;
  require Image::Base::GD;
  my $image = Image::Base::GD->new (-width => $width, -height => $height);
  $image->rectangle(0,0, $width-1,$height-1, 'black');

  require Geometry::AffineTransform;
  my $affine = Geometry::AffineTransform->new;

  $affine->scale(1, -1);   # Y mirror
  $affine->translate(0, $height);   # Y mirror

  my @colour = ('white') x ($depth_limit+1);
  # $colour[4] = 'red';
  # my $h = 255 - ($depth-1) / $depth_limit * 255;
  # my $colour = sprintf '#%02X%02X%02X', $h,$h,$h;

  my $draw_at_n = sub {
    my ($n, $x1,$y1, $colour) = @_;
    my $depth = $path->tree_n_to_depth($n);

    foreach my $to_n ($n+1, $n-1) {
      my $to_depth = $path->tree_n_to_depth($to_n);
      next unless $depth == $to_depth;

      my ($x2,$y2) = $path->n_to_xy($to_n);
      next unless $y1 == $y2;
      if (ref $x2) { $x2 = $x2->numify; }
      if (ref $y2) { $y2 = $y2->numify; }

      {
        my $x1 = $x1 * $scale;
        my $y1 = $y1 * $scale;
        my $x2 = $x2 * $scale;
        my $y2 = $y2 * $scale;

        my $offset = min($depth, 3);
        $y1 += $offset;
        $y2 += $offset;

        ($x1,$y1) = $affine->transform($x1,$y1);
        ($x2,$y2) = $affine->transform($x2,$y2);

        $colour ||= $colour[$depth]||'white';
        $image->line ($x1,$y1, $x2,$y2, $colour);
      }
    }
  };

  foreach my $y1 (0 .. $y_limit) {
    foreach my $x1 (0 .. $x_limit) {
      my $n = $path->xy_to_n(Math::BigInt->new($x1),Math::BigInt->new($y1))
        // next;
      next if $n == $n_start;
      $draw_at_n->($n, $x1,$y1);
    }
  }

  my ($n_lo, $n_hi) = $path->tree_depth_to_n_range(4);
  foreach my $n ($n_lo .. $n_hi) {
    my ($x1,$y1) = $path->n_to_xy($n);
    $draw_at_n->($n, $x1,$y1, 'red');
  }

  $image->save('/tmp/x.png');
  require IPC::Run;
  IPC::Run::run(['xzgv','/tmp/x.png']);
  exit 0;
}



{
  # A,B,C modulo 3
  # initial 012  all 012,022,101,201
  # U   012 -> 201 -> 101 -> 012cycle     022->022cycle
  # A   012 -> 022->022cycle              101->012->022->022cycle
  # D   012 -> 022 -> 201 -> 012cycle     101->101cycle

  my $tree_type = 'FB';
  my $modulus = 3;
  my $path_AB = Math::PlanePath::PythagoreanTree->new (tree_type => $tree_type,
                                                       coordinates => 'AB',
                                                       # digit_order => 'LtoH',
                                                      );
  my $path_AC = Math::PlanePath::PythagoreanTree->new (tree_type => $tree_type,
                                                       coordinates => 'AC',
                                                       # digit_order => 'LtoH',
                                                      );
  my %seen;
  foreach my $depth (0 .. 8) {
    print "\ndepth $depth\n";
    my ($n_lo, $n_hi) = $path_AB->tree_depth_to_n_range($depth);
    foreach my $n ($n_lo .. $n_hi) {
      my ($A,$B) = $path_AB->n_to_xy($n);
      (undef, my $C) = $path_AC->n_to_xy($n);
      my $str = ($A % $modulus) . ($B % $modulus) . ($C % $modulus);
      if ($depth <= 3) {
        print "$str\n";
      }
      $seen{$str} = 1;
    }
  }
  $, = ' ';
  print sort keys %seen, "\n";
  exit 0;
}

{
  # UArD horizontal lengths

  require Math::BigInt;
  Math::BigInt->import (try => 'GMP');
  require Set::IntSpan::Fast;
  my $x_limit = 41;
  my $y_limit = 40;

  require Math::PlanePath::PythagoreanTree;
  my $path = Math::PlanePath::PythagoreanTree->new (tree_type => 'UArD',
                                                    coordinates => 'PQ');
  my $n_start = $path->n_start;

  {
    print "horizontals\n";
    foreach my $y1 (0 .. $y_limit) {
      my %seen;
      print "Y=$y1\n";
      foreach my $x1 (0 .. $x_limit) {
        my $n = $path->xy_to_n(Math::BigInt->new($x1),Math::BigInt->new($y1))
          // next;
        next if $n == $n_start;
        my $depth = $path->tree_n_to_depth($n);
        ### point: "$x1,$y1  depth=$depth"

        foreach my $to_n ($n+1, $n-1) {
          my $to_depth = $path->tree_n_to_depth($to_n);
          next unless $depth == $to_depth;

          my ($x2,$y2) = $path->n_to_xy($to_n);
          next unless $y1 == $y2;
          $x2 = $x2->numify;
          $y2 = $y2->numify;

          if ($x1 > $x2) { ($x1,$x2) = ($x2,$x1); }
          next if $seen{$x1}++;

          my $len = $x2 - $x1 + 1;
          print "$x1..$x2 length $len\n";
        }
      }
    }
  }
  exit 0;
}


{
  # UAD, FB triples in identical positions, only 5,12
  my $path = Math::PlanePath::PythagoreanTree->new;
  my $pfb = Math::PlanePath::PythagoreanTree->new (tree_type => 'FB');
  for (my $n = $path->n_start; $n < 100000; $n++) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($fx,$fy) = $pfb->n_to_xy($n);
    if ($x == $fx && $y == $fy) {
      print "$n  $x,$y\n";
    }
  }
  exit 0;
}

{
  # UAD A/B ratio
  my $path = Math::PlanePath::PythagoreanTree->new;
  foreach my $child (0,1,2) {
    print "child $child\n";
    my $max = 0;
    for (my $n = $path->n_start+1 + $child; $n < 100000; $n += 3) {
      my ($A,$B) = $path->n_to_xy($n);
      my $ratio = $A/$B;
      if ($ratio > $max) {
        $max = $ratio;
        print "  $n $ratio\n";
      }
    }
  }
  exit 0;
}


{
  require Math::Geometry::Planar;
  #my $points = [ [0,0], [1,1], [2,2], [1,1] ];
  my $points = [ [0,0], [1,1] ];
  my $planar = Math::Geometry::Planar->new;
  $planar->points($points);
  my $area = $planar->area;
  print "area $area\n";
  exit 0;
}

{
  # UAD A-B row total +1,-1
  #     A = p^2 - q^2
  #     B = 2pq
  #     A-B = p^2 - 2pq - q^2
  #         = p(p-q) - pq - q^2
  #         = p(p-q) - q(p+q)
  # M = U+A+D = 1  2  6
  #             2  1  6
  #             2  2  9
  # A' =  A + 2B + 6C
  # B' = 2A +  B + 6C
  # C' = 2A + 2B + 9C
  #
  # A' - B' = -A + B
  #
  # (A0) = (1 0 0) ABC
  # ( 0)   (0 0 0)
  # ( 0)   (0 0 0)
  #
  # ( 0) = (0 0 0) M.ABC
  # (A1)   (1 0 0)
  # ( 0)   (0 0 0)
  #
  # ( 0) = (0 0 0) M^2.ABC
  # ( 0)   (0 0 0)
  # (A2)   (1 0 0)
  #
  # (A0) = (1 0 0) ABC   (0 0 0) M.ABC    (0 0 0) M^2.ABC
  # (A1)   (0 0 0)     + (1 0 0)        + (0 0 0)
  # (A2)   (0 0 0)       (0 0 0)          (1 0 0)
  #
  # m = u+a+d
  # abc_to_a012 = [1,0,0;0,0,0;0,0,0] + [0,0,0;1,0,0;0,0,0] * m + [0,0,0;0,0,0;1,0,0] * m^2
  # = [1 0 0]
  #   [1 2 6]
  #   [17 16 72]

  # A012 = abc_to_a012 * ABC
  # A123 = abc_to_a012 * m * abc_to_a012^-1 * A012
  # am = abc_to_a012 * m * abc_to_a012^-1
  #    = [0 1 0]
  #      [0 0 1]
  #      [-3 9 11]
  # A3 = -3*A0 + 9*A1 + 11*A2
  #  -3*3 + 9*45 + 11*475

  print U_3x3() + A_3x3() + D_3x3();
  print M1_3x3() + M2_3x3() + M3_3x3();
  print U_3x3() + M2_3x3() + T_3x3();

  # tree row totals

  my $path = Math::PlanePath::PythagoreanTree->new;
  my $path_ac = Math::PlanePath::PythagoreanTree->new (coordinates => 'AC');
  my @A_total;
  my @B_total;
  my @C_total;
  my @sum_total;
  my @diff_total;
  foreach my $depth (0 .. 7) {
    my $A_total;
    my $B_total;
    my $C_total;
    my $sum_total;
    my $diff_total;
    my ($n_row, $n_end) = $path->tree_depth_to_n_range($depth);
    foreach my $n ($n_row .. $n_end) {
      my ($A,$B) = $path->n_to_xy($n);
      my ($AA,$C) = $path_ac->n_to_xy($n);
      $A_total += $A;
      $B_total += $B;
      $C_total += $C;
      $sum_total += $A + $B;
      $diff_total += $A - $B;
    }
    push @A_total, $A_total;
    push @B_total, $B_total;
    push @C_total, $C_total;
    push @sum_total, $sum_total;
    push @diff_total, $diff_total;

    my $A_formula = uad_A_total_by_recurrence($depth);
    print "$depth   $A_total,$B_total,$C_total  $A_formula\n";
  }
  print "X rows\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@A_total);
  print "\n";

  print "Y rows\n";
  Math::OEIS::Grep->search(array => \@B_total);
  print "\n";

  print "C rows\n";
  Math::OEIS::Grep->search(array => \@C_total);
  print "\n";

  print "X+Y rows\n";
  Math::OEIS::Grep->search(array => \@sum_total);
  print "\n";

  print "X-Y rows\n";
  Math::OEIS::Grep->search(array => \@diff_total);
  print "\n";
  exit 0;

  # A3 = -3*A0 + 9*A1 + 11*A2
  sub uad_A_total_by_recurrence {
    my ($depth) = @_;
    if ($depth == 0) { return 3; }
    if ($depth == 1) { return 41; }
    if ($depth == 2) { return 475; }
    return -3*uad_A_total_by_recurrence($depth-3)
      + 9*uad_A_total_by_recurrence($depth-2)
        +  11*uad_A_total_by_recurrence($depth-1);
  }
}
{
  # row area of RationalsTree
  # between: 0.5, 3,8,18,38,78,158,318,638

  # L = (1 1)   R = (1 0)
  #     (0 1)       (1 1)
  #
  # r[n+1] = 2*r[n] + gap
  #
  # L^n * (1) = (n+1)   R^n * (1) = ( 1 )
  #       (1)   ( 1 )         (1)   (n+1)
  #
  # L.R^n.[1;1] = [n+2; n+1]   L.R^(n+1).[1;1] = [n+3; n+2]
  # R.L^n.[1;1] = [n+1; n+2]   R.L^(n+1).[1;1] = [n+2; n+3]
  #
  #         +2,+3 *             area = 2*2/2 = 2
  #              / \
  #             /   \
  #     +1,+2  *     * +3,+2
  #             \   /
  #              \ /
  #               *
  #             +2,+1
  #
  # r[n+1] = 2*r[n] + 2
  #   3 == 2*.5 + 2
  #   8 == 2*3 + 2
  #   18 == 2*8 + 2
  #   38 == 2*18 + 2

  # r[n] = 2 + 2*(2 + 2*(2 + 2*(... 0.5)))
  #      = 2*(1 + .. + 2^(n-1)) + 2^(n-1)*.5
  #      = 2*(2^n - 1) + 2^(n-1)
  #      = 4*2^(n-1) - 2 + 2^(n-1)
  #      = 5*2^(n-1) - 2
  # for(n=0,8,print(5*2^(n-1) - 2,","))
  #
  # Cumulative
  # R[n] = sum r[0] to r[n-1]
  #      = 5*(2^-1 + 1 + ... + 2^(n-2)) - 2n
  #      = 5/2 + 5*(2^(n-1) - 1) - 2n
  #      = 5*(1/2 + 2^(n-1) - 1) - 2n
  #      = 5*(2^(n-1) - 1/2) - 2n
  #      = 5/2*(2^n - 1) - 2n
  #
  require Math::PlanePath::RationalsTree;
  require Math::Geometry::Planar;
  my $path = Math::PlanePath::RationalsTree->new;
  my @values;
  my @prev_points = ([1,1]);
  my $total = 0;
  foreach my $depth (1 .. 9) {
    my ($n_row, $n_end) = $path->tree_depth_to_n_range($depth);
    my @points = map {[$path->n_to_xy($_)]} $n_row .. $n_end;

    my $planar = Math::Geometry::Planar->new;
    my $points = [ @points, reverse @prev_points ];
    $planar->points($points);
    my $area = $planar->area;
    $total += $area;

    my $area_formula = rationals_area_between_by_formula($depth-1);
    my $total_formula = rationals_total_by_formula($depth);

    print "$depth  $area $area_formula    $total $total_formula\n";
    if (@points <= 10) {
      print "   ",points_str($points),"\n";
    }
    @prev_points = @points;

    push @values, $total*2;
  }

  shift @values;
  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values);
  exit 0;

  sub rationals_area_between_by_formula {
    my ($depth) = @_;
    return 5*2**($depth-2) - 2;
  }
  sub rationals_total_by_formula {
    my ($depth) = @_;
    return (2**$depth - 1) * 5/2 - 2*$depth;
  }
}

{
  # matrices odd/even terms
  #
  # det = ad-bc
  #     = (a mod 4).(d mod 4) - (b mod 4)(c mod 4)

  my $limit = 20;
  foreach my $want_EO_str (
                           'EE,OO',
                           # 'OE,EO',
                           # 'EO,OE',
                          ) {
    print "$want_EO_str\n";
    foreach my $a (1 .. $limit) {          # a >= 1
      foreach my $b (1-$a .. $limit) {     # a+b >= 1    so b >= 1-a
        foreach my $c (0 .. $a-1) {    # a > c >= 0
          foreach my $d (-$c .. $limit) {  # c+d >= 0 so d >= -c
            next if abcd_conditions_fail($a,$b,$c,$d);
            my $EO_str = abcd_EO_str($a,$b,$c,$d);

            if ($EO_str eq $want_EO_str) {
              my $det = $a*$d - $b*$c;

              my $ad = 0;
              if ($EO_str eq 'OE,EO') {
                $ad = ($a*$d) % 4;
              } elsif ($EO_str eq 'EO,OE') {
                $ad = (-$b*$c) % 4;
              }
              if ($ad == 3) { $ad = -1; }
              my $diff = ($ad == $det ? '' : '  ***');

              print "$a,$b,$c,$d   $det $ad$diff\n";
            }
          }
        }
      }
      print "\n";
    }
  }
  exit 0;

  sub abcd_EO_str {
    my ($a,$b,$c,$d) = @_;
    return EO_str($a).EO_str($b).','.EO_str($c).EO_str($d);
  }
  sub EO_str {
    my ($n) = @_;
    return ($n & 1 ? 'O' : 'E');
  }
}


{
  # parallelograms row 2 to 3

  my $path = Math::PlanePath::PythagoreanTree->new(coordinates => 'PQ',
                                                   digit_order => 'LtoH');

  my $draw_line = sub {
    my ($x1,$y1, $x2,$y2) = @_;
    print "  \\draw ($x1, $y1) -- ($x2, $y2);\n";
  };

  my $draw_poly = sub {
    my @coords = @_;
    print "  \\filldraw ";
    while (@coords) {
      my $x = shift @coords;
      my $y = shift @coords;
      print "($x, $y)--";
    }
    print "cycle;\n";
  };

  my $depth = 1;
  foreach my $n ($path->tree_depth_to_n($depth)
                 .. $path->tree_depth_to_n_end($depth)) {
    my ($x,$y) = $path->n_to_xy($n);

    my ($n_u, $n_a, $n_d) = $path->tree_n_children($n);
    my ($x_u,$y_u) = $path->n_to_xy($n_u);
    my ($x_a,$y_a) = $path->n_to_xy($n_a);
    my ($x_d,$y_d) = $path->n_to_xy($n_d);

    $draw_poly->($x,$y,
                 $x_u,$y_u,
                 $x_a,$y_a,
                 $x_d,$y_d);
  }
  exit 0;
}

{
  # UArD under 9,2 to 8,5
  my $path = Math::PlanePath::PythagoreanTree->new(tree_type => 'UArD',
                                                   coordinates => 'PQ');
  foreach my $parent_n ($path->xy_to_n(9,2) .. $path->xy_to_n(8,5)) {
    my ($parent_x,$parent_y) = $path->n_to_xy($parent_n);
    print "$parent_x,$parent_y";
    foreach my $n ($path->tree_n_children($parent_n)) {
      my ($x,$y) = $path->n_to_xy($n);
      print "    $x,$y";
    }
    print "\n";
  }

  exit;
}
{
  # UArD row totals
  #
  # p = 2,12,70,408,2378,13860,80782,470832
  # A001542 a(n) = 6a(n-1) - a(n-2), with a(0) = 0 and a(1) = 2.
  # Even Pell numbers
  #
  # q = 1,5,29,169,985,5741,33461,195025
  # A001653 Numbers n such that 2*n^2 - 1 is a square.
  # Odd Pell numbers
  #
  # H=2Q
  #
  # S = sum i=0 to i=depth-1 of P-Q
  #   = sum even Pell - odd Pell

  my $path = Math::PlanePath::PythagoreanTree->new
    (tree_type => 'UArD',
     coordinates => 'PQ');
  my @values;
  my $P_total = 0;
  my $Q_total = 0;
  my $PQ_diff_total = 0;
  foreach my $depth (0 .. 6) {
    my $n_row = $path->tree_depth_to_n($depth);
    my $n_end = $path->tree_depth_to_n_end($depth);
    my @XY = (0, 0);
    my $P = 0;
    my $Q = 0;
    foreach my $n ($n_row .. $n_end) {
      my ($x,$y) = $path->n_to_xy($n);
      $P += $x;
      $Q += $y;
    }
    my $H = 0;
    my $S = 0;
    foreach my $n ($n_row .. $n_end-1) {
      my ($dx,$dy) = $path->n_to_dxdy($n);
      if ($dy == 0) {
        $H += abs($dx);
      } else {
        $S += abs($dx);
      }
    }
    my $PQ_diff = $P - $Q;
    $P_total += $P;
    $Q_total += $Q;
    $PQ_diff_total += $PQ_diff;

    my $SS = UArD_diag_calc($depth);
    $PQ_diff_total = $P_total - $Q_total;
    my $Q2 = $Q*2;
    my $Q2_total = $Q_total*2;
    push @values, $S;

    # unless ($depth & 1) { $S /= 2; }
    # $S = sqrt($S);

    my ($Pr,$Qr) = UAD_pq_total_by_recurrence($depth);
    $Pr == $P or die;
    $Qr == $Q or die;

    print "$depth  $H $S,$SS   P=$P 2Q=$Q2 P-Q=$PQ_diff total  $P_total 2Q=$Q2_total P-Q=$PQ_diff_total\n";
  }
  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values);
  exit 0;

  sub UArD_diag_calc {
    my ($depth) = @_;

    {
      require Math::NumSeq::Pell;
      my $ret = 0;
      foreach my $i (0 .. $depth-1) {
        my ($P,$Q) = UAD_pq_total_by_recurrence($i);
        $ret += $P-$Q;
      }
      return $ret;
    }
    {
      require Math::NumSeq::Pell;
      my $ret = 0;
      foreach my $i (1 .. $depth) {
        $ret += Math::NumSeq::Pell->ith(2*$i) - Math::NumSeq::Pell->ith(2*$i-1);
      }
      return $ret;
    }

    my $s0 = 0;
    my $s1 = 1;
    foreach (1 .. $depth) {
      ($s1,$s0) = (6*$s1-$s0+2, $s1);
    }
    return $s0;
  }
}




{
  # All row total p,q searches
  #
  # p = 2,12,70,408,2378,13860,80782,470832
  # A001542 a(n) = 6a(n-1) - a(n-2), with a(0) = 0 and a(1) = 2.
  # Even Pell numbers
  #
  # q = 1,5,29,169,985,5741,33461,195025
  # A001653 Numbers n such that 2*n^2 - 1 is a square.
  # Odd Pell numbers
  #
  # A000129 Pell numbers

  require Math::BigInt;
  Math::BigInt->import (try => 'GMP');
  my $coordinates_choices = Math::PlanePath::PythagoreanTree
    ->parameter_info_hash->{'coordinates'}->{'choices'};

  foreach my $tree_type ('UAD', 'FB', 'UMT') {
    my %seen_coord_name;
    foreach my $coordinates (@$coordinates_choices) {
      foreach my $coord (1 .. 6) {
        my $coord_name;
        if ($coord == 1) { $coord_name = substr($coordinates,0,1); }
        if ($coord == 2) { $coord_name = substr($coordinates,1,1); }
        if ($coord == 3) { $coord_name = $coordinates; }
        if ($coord == 4) { $coord_name = (substr($coordinates,1,1)
                                          . substr($coordinates,0,1)); }
        if ($coord == 5) { $coord_name = (substr($coordinates,0,1)
                                          . '-' . substr($coordinates,1,1)); }
        if ($coord == 6) { $coord_name = ('abs '.substr($coordinates,0,1)
                                          . '-' . substr($coordinates,1,1)); }
        next if $seen_coord_name{$coord_name}++;
        print "\n$tree_type $coord_name\n";

        my $path = Math::PlanePath::PythagoreanTree->new
          (tree_type => $tree_type,
           coordinates => $coordinates);
        my @values;
        foreach my $depth (0 .. 6) {
          my $n_row = $path->tree_depth_to_n($depth);
          my $n_end = $path->tree_depth_to_n_end($depth);
          my @XY = (0, 0);
          foreach my $n ($n_row .. $n_end) {
            my ($x,$y) = $path->n_to_xy(Math::BigInt->new($n));
            $XY[0] += $x;
            $XY[1] += $y;
          }
          my ($pr,$qr) = UAD_pq_total_by_recurrence($depth);
          my $pp = UAD_p_total_by_recurrence($depth);
          my $qq = UAD_q_total_by_recurrence($depth);
          print "$depth  $XY[0] $XY[1]   $pr,$pp $qr,$qq\n";

          if ($coord == 1) {
            push @values, $XY[0];
          }
          if ($coord == 2) {
            push @values, $XY[1];
          }
          if ($coord == 3) {
            push @values, $XY[0];
            push @values, $XY[1];
          }
          if ($coord == 4) {
            push @values, $XY[1];
            push @values, $XY[0];
          }
          if ($coord == 5) {
            push @values, $XY[0] - $XY[1];
          }
          if ($coord == 6) {
            push @values, abs($XY[0] - $XY[1]);
          }
        }
        require Math::OEIS::Grep;
        Math::OEIS::Grep->search(array => \@values);
      }
    }
  }
  exit 0;

  # U+A+D = (5 2)
  #         (2 1)
  # p2 = 5*p1 + 2*q1
  # q2 = 2*p1 +   q1
  #
  # p2 - 2*q2 = p1
  # 2*q2 = p2 - p1
  # p2 = 5*p1 + p1 - p0 = 6*p1 - p0
  #
  # 5*q2 - 2*p2 = 5*q1 - 4*q1 = q1
  # 2*p2 = 5*q2 - q1
  # q2 = 5*q1 - q0 + q1 = 6*q1 - q0
  #
  # 12 = 6*2 - 0   p[-1] = 0
  # 5 = 6*1 - 1    q[-1] = 1
  #
  # a(n+1) = 2*a(n) + a(n-1)
  #        = 2*(2*a(n-1) + a(n-2)) + a(n-1)
  #        = 5*a(n-1) + a(n-2)
  # a(n)   = 2*a(n-1) + a(n-2)
  #
  sub UAD_pq_total_by_recurrence {
    my ($depth) = @_;
    my $p = 2;
    my $q = 1;
    foreach (1 .. $depth) {
      ($p,$q) = (5*$p+2*$q, 2*$p+$q);
    }
    return ($p,$q);
  }
  sub UAD_p_total_by_recurrence {
    my ($depth) = @_;
    my $p0 = 2;
    my $p1 = 3+5+4;
    foreach (1 .. $depth) {
      ($p1,$p0) = (6*$p1 - $p0, $p1);
    }
    return $p0;
  }
  sub UAD_q_total_by_recurrence {
    my ($depth) = @_;
    my $q0 = 1;
    my $q1 = 2+2+1;
    foreach (1 .. $depth) {
      ($q1,$q0) = (6*$q1 - $q0, $q1);
    }
    return $q0;
  }
}

{
  # row length by alpha,beta
  # want h 2,12,70,408,2378,13860,80782
  #      d 1,8,49,288,1681,9800,57121
  my $alpha = 2;
  my $beta = 1;
  my $h = 0;
  my $s = 0;
  foreach my $depth (1 .. 7) {
    $h += $alpha;
    $s += $beta;
    print "$depth  $alpha $beta    $h $s\n";
    ($alpha,$beta) = (3*$alpha + 4*$beta,
                      2*$alpha + 3*$beta);
  }
  exit 0;
}

{
  # UArD q values list
  my $path = Math::PlanePath::PythagoreanTree->new(tree_type => 'UArD',
                                                   coordinates => 'PQ',
                                                   digit_order => 'HtoL');
  my @values;
  foreach my $depth (0 .. 4) {
    my $n_row = $path->tree_depth_to_n($depth);
    my $n_end = $path->tree_depth_to_n_end($depth);
    foreach my $n ($n_row .. $n_end-1) {
      my ($x,$y) = $path->n_to_dxdy($n);
      $x = abs($x);
      print "$x,";
    }
    print "\n";
    foreach my $n ($n_row .. $n_end-1) {
      my ($x,$y) = $path->n_to_dxdy($n);
      $y = abs($y);
      if ($y != 0) { print "$y,"; }
    }
    print "\n";
  }
  exit 0;
}
{
  # UArD row length

  # U rotate     *        *---.---*
  #   shear      |  ->                 len *= 2
  #              *
  #                   p,q+k         p+2(q+k),q+k
  #                     *          __*         p += 2q
  #                 ->  |       .--
  #          *---*      *      *
  #                   p,q   p+2q,q
  #
  # h^2 = (p+2(q+k) - (p+2q))^2 + (q+k - q)^2
  #     = (p+2q + 2k - p-2q)^2 + k^2
  #     = (2k)^2 + k^2
  #     = 5*k^2
  # len *= sqrt(5)

  # UArD HtoL
  # horiz 2,12,70,408,2378,13860,80782
  # = A001542 a(n) = 6a(n-1) - a(n-2), with a(0) = 0 and a(1) = 2.
  #
  # diag 1,8,49,288,1681,9800,57121
  # A001108 a(n)-th triangular number is a square: a(n+1) = 6*a(n)-a(n-1)+2,
  #         with a(0) = 0, a(1) = 1.
  #
  # total 3,20,119,696,4059,23660,137903
  # A001652 a(n) = 6*a(n-1) - a(n-2) + 2 with a(0) = 0, a(1) = 3.


  require List::MoreUtils;
  my $path = Math::PlanePath::PythagoreanTree->new(tree_type => 'UArD',
                                                   coordinates => 'PQ',
                                                   digit_order => 'HtoL');
  my @values;
  foreach my $depth (0 .. 7) {
    my $dist = 0;
    my $n_row = $path->tree_depth_to_n($depth);
    my $n_end = $path->tree_depth_to_n_end($depth);
    my ($x_prev,$y_prev) = $path->n_to_xy($n_row);
    my $prev_atan = atan2($y_prev,$x_prev);
    my %steps;
    require Tie::IxHash;
    tie %steps, 'Tie::IxHash';
    foreach my $n ($n_row+1 .. $n_end) {
      my ($x,$y) = $path->n_to_xy($n);
      my $dx = abs($x - $x_prev);
      my $dy = abs($y - $y_prev);
      $steps{"$dx,$dy"}++;
      my $h = $dx*$dx + $dy*$dy;

      my ($square, $rem) = square_split($h);
      # $steps{$rem} += $square;

      if ($dy == 0) { $dist += abs($dx); }   # horiz
      if ($dx == $dy) { $dist += abs($dx); }   # diag
      # $h = hypot($dx,$dy);
      # $dist += $h;

      my $atan = atan2($y,$x);
      # if ($atan > $prev_atan) { die; } # UArD LtoH is clockwise

      $x_prev = $x;
      $y_prev = $y;
      $prev_atan = $atan;
    }

    my @steps = keys %steps;
    # @steps = sort {$a<=>$b} @steps;
    my @mults = map{$steps{$_}} @steps;
    print "\nsteps ",join('  ',@steps),"\n";
    print "mults ",join('  ',@mults),"\n";

    print "$depth dist=$dist\n";
    push @values, $dist;

    next;
    if (@steps > 5) {
      require Math::OEIS::Grep;
      # Math::OEIS::Grep->search(array => \@steps);

      Math::OEIS::Grep->search(array => \@mults);
      @mults = reverse @mults;
      Math::OEIS::Grep->search(array => \@mults);
      @mults = sort {$a<=>$b} @mults;
      Math::OEIS::Grep->search(array => \@mults);

      # @mults = List::MoreUtils::uniq(@mults);
      # print "mults distinct ",join(',',@mults),"\n";
      # Math::OEIS::Grep->search(array => \@mults);
    }
  }

  shift @values;

  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values);
  exit 0;


  sub square_split {
    my ($n) = @_;
    my $square = 1;
    foreach my $i (2 .. sqrt($n)+1) {
      if ($n % ($i*$i) == 0) {
        $square *= $i;
        $n /= $i*$i;
      }
    }
    return ($square, $n);
  }
}

{
  # UArD high to low horiz or diagonal
  my $path = Math::PlanePath::PythagoreanTree->new(coordinates => 'PQ',
                                                   tree_type => 'UArD');
  foreach my $depth (0 .. 5) {
    my $n_row = $path->tree_depth_to_n($depth);
    foreach my $n ($n_row .. $path->tree_depth_to_n_end($depth) - 1) {
      my $offset = $n - $n_row;
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my $str = ($dx > 0 && $dy == 0 ? "horiz right"
                 : $dx < 0 && $dy == 0 ? "horiz left"
                 : $dx == $dy ? "diag NE"
                 : $dx == -$dy ? "diag SW"
                 : "");
      my $l = lowest_ternary_nonzero($offset+1);
      printf "%3d %3d,%3d   %s %s\n", $n, $dx,$dy, $str, $l;
    }
    print "\n";
  }
  exit 0;

  sub lowest_ternary_nonzero {
    my ($n) = @_;
    while ($n % 3 == 0) {
      $n /= 3;
      $n or return 0;
    }
    return $n % 3;
  }
}

{
  # UArD vs UAD row lines SVG image

  my $depth = 2;
  my $yfactor = 2;

  my $scale;
  {
    my $max = 0;
    my $path = Math::PlanePath::PythagoreanTree->new(coordinates => 'PQ',
                                                     tree_type => 'UAD');
    foreach my $n ($path->tree_depth_to_n($depth+1) .. $path->tree_depth_to_n_end($depth+1)) {
      my ($x,$y) = $path->n_to_xy($n);
      $max = max($max, $x, $y*$yfactor);
    }
    $scale = 800 / $max * .95;
  }
  print "scale $scale\n";
  my $xscale = $scale;
  my $yscale = $scale * -$yfactor;

  eval "use SVG; 1" or die;
  my $svg = SVG->new(width => 800, height => 800);
  $svg->rectangle(x=>0, y=>0, width=>800, height=>800,
                  style => {'fill'           => 'black',
                            'stroke'         => 'black',
                            'stroke-opacity' =>  .5,
                            'fill-opacity'   =>  1 });

  my $group;
  my $draw = sub {
    my ($tree_type) = @_;
    my $path = Math::PlanePath::PythagoreanTree->new(coordinates => 'PQ',
                                                     tree_type => $tree_type);
    foreach my $d (0 .. $depth) {
      my (@x,@y);
      foreach my $n ($path->tree_depth_to_n($d)
                     .. $path->tree_depth_to_n_end($d)) {
        my ($x,$y) = $path->n_to_xy($n);
        push @x, $x;
        push @y, $y;
        $group->circle(cx=>$x,cy=>$y, r=>.1,
                       style => {'stroke-width' => 0,
                                 fill => 'white',
                                });
      }

      foreach my $i (0 .. $#x-1) {
        $group->line(x1=>$x[$i], y1=>$y[$i],
                     x2=>$x[$i+1], y2=>$y[$i+1],
                     style => { stroke => 'white',
                                'stroke-width' => .05,
                                'stroke-linecap' => 'round',
                              });
      }

      # my $points = $svg->get_path(x=>\@x, y=>\@y, -type=>'polygon');
      # $group->polyline(%$points,
      #                  style => { stroke => 'white',
      #                             'stroke-width' => .05,
      #                             'stroke-linecap' => 'round',
      #                           },
      #                 );
    }
  };

  $group = $svg->group(transform =>
                       "translate(0,800) scale($xscale,$yscale)");
  $draw->('UAD');
  $group = $svg->group(transform =>
                       "translate(400,800) scale($xscale,$yscale)");
  $draw->('UArD');

  my $xml = $svg->xmlify;
  open FH, '>', '/tmp/x.svg';
  print FH "$xml\n";
  close FH;
  ### $xml
  require IPC::Run;
  # IPC::Run::run (['see','--debug','image/svg+xml:-'], '<', \$xml);
  IPC::Run::run (['see','--debug','/tmp/x.svg'], '<', \$xml);
  exit 0;
}




{
  # row length by matrix mults
  #
  # dist -> 5.82842712501088 ^ depth
  # sum dist^2 -> 11.7445626465381 ^ depth
  #
  my $path = Math::PlanePath::PythagoreanTree->new(tree_type => 'UArD',
                                                   coordinates => 'PQ',
                                                   digit_order => 'LtoH');
  my $steps = {};
  my $prev_log = 0;
  foreach my $depth (0 .. 20) {
    ### $depth
    my %new_steps;
    my $dist = 0;
    my ($str, $mul);
    my $push = sub {
      my ($new_dx, $new_dy) = @_;
      while ($new_dx % 4 == 0 && $new_dy % 4 == 0) {
        $mul *= 2;
        $new_dx /= 2;
        $new_dy /= 2;
      }
      while ($new_dx % 9 == 0 && $new_dy % 9 == 0) {
        $mul *= 3;
        $new_dx /= 3;
        $new_dy /= 3;
      }
      $new_steps{"$new_dx,$new_dy"} += $mul;
      # $dist += $mul * ($new_dx**2 + $new_dy**2);
      $dist += $mul * hypot($new_dx,$new_dy);
      ### dist: "$new_dx, $new_dy   $mul"
    };
    $mul = 1;
    $push->(2,0);
    $push->(1,1);
    while (($str, $mul) = each %$steps) {
      my ($dx,$dy) = split /,/, $str;
      $push->(2*$dx-$dy, $dx);
      $push->(2*$dx+$dy, $dx);
      $push->($dx+2*$dy, $dy);
    }
    $steps = \%new_steps;
    my $f = $dist / exp(1.76274717403824)**$depth;
    my $log = log($dist); #  / ($depth||1);
    my $dlog = $log - $prev_log;
    my $elog = exp($dlog);
    print "$depth  $dist   $f $log $dlog $elog\n";
    $prev_log = $log;
  }
  exit 0;
}

{
  # row steps
  my $path = Math::PlanePath::PythagoreanTree->new(tree_type => 'UArD',
                                                   coordinates => 'PQ',
                                                   digit_order => 'LtoH');
  foreach my $depth (0 .. 4) {
    my $rep = 3**($depth-1) - 1;
    my $n_row = $path->tree_depth_to_n($depth);
    foreach my $n ($n_row .. $path->tree_depth_to_n_end($depth) - 1) {
      if ($n - $n_row == $rep
          || $n - $n_row == $rep+1
          || $n - $n_row == 2*$rep+1
          || $n - $n_row == 2*$rep+2) {
        print "--";
      } else {
        print " ";
      }
      my ($dx,$dy) = $path->n_to_dxdy($n);
      $dx = abs($dx);
      $dy = abs($dy);
      my $g = gcd($dx,$dy);
      $dx /= $g;
      $dy /= $g;
      print "$dx,$dy";
    }
    print "\n";
  }
  exit 0;
}
{
  # lower parallelogram
  require Math::Geometry::Planar;
  my $root = Math::Matrix->new([2],[1]);
  my $start_l = D_2x2() * $root;
  my $start_r = A_2x2() * $root;
  ### $start_l
  ### $start_r

  my $vtop = $root;
  my $vbot = $root;
  foreach my $i (0 .. 5) {
    ### $vtop
    ### $vbot
    my $v_l = D_2x2() * $vtop;
    my $v_r = A_2x2() * $vtop;
    my $v_l_str = mvec_to_str($v_l);
    my $v_r_str = mvec_to_str($v_r);

    my $planar = Math::Geometry::Planar->new;
    my $points = [[mvec_to_pq($start_l)],
                  [mvec_to_pq($v_l)],
                  [mvec_to_pq($v_r)],
                  [mvec_to_pq($start_r)]];
    ### $points
    $planar->points($points);
    my $a = $planar->area;
    print "$i  $v_l_str $v_r_str area=$a\n";

    $vtop = U_2x2() * $vtop;
    $vbot = D_2x2() * $vbot;
  }
  exit 0;
}

{
  # ternary_1s_reverse() is A128173 ternary reflected Gray code
  my @values = map {ternary_1s_reverse($_)} 1 .. 30;
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values);

  my $anum = 'A128173';
  my $seq = Math::NumSeq::OEIS::File->new (anum => $anum);
  my $adjust = $seq->i_start - $seq->ith($seq->i_start);
  my $count = 0;
  while (my ($i,$value) = $seq->next) {
    $i -= $adjust;
    my $ternary_1s_reverse = ternary_1s_reverse($i);
    $ternary_1s_reverse == $value
      or die "ternary_1s_reverse i=$i got $ternary_1s_reverse want $value from $anum";
    $count++;
  }
  print "ternary_1s_reverse() ok on $count values\n";
}


{
  # possible matrices

  require Math::PlanePath::GcdRationals;
  *gcd = \&Math::PlanePath::GcdRationals::_gcd;

  # 1
  # 1 + 3 = 4
  # 1 + 3 + 9 = 13
  # 1 + 3 + 9 + 27 = 40
  my @p;
  my @q;
  my @pq;
  {
    my $path = Math::PlanePath::PythagoreanTree->new(coordinates => 'PQ');
    foreach my $i (0 .. 120) {
      ($p[$i],$q[$i]) = $path->n_to_xy($path->n_start + $i);
      $pq[$i] = "$p[$i],$q[$i]";
    }
    $#pq = 13;
  }
  # {
  #   foreach my $p (2 .. 18) {
  #     foreach my $q (1 .. 18) {
  #       next unless pq_acceptable($p,$q);
  #       push @p, $p;
  #       push @q, $q;
  #       push @pq, "$p,$q";
  #     }
  #   }
  # }

  my $term_min = -4;
  my $term_max = 4;
  my @term_list;
  for (my $term = $term_min; $term <= $term_max; $term += 0.5) {
    push @term_list, $term;
  }
  my @matrices;
  foreach my $a (@term_list) {
    foreach my $b (@term_list) {
      foreach my $c (@term_list) {
        foreach my $d (@term_list) {
          my $m = [$a,$b,$c,$d];
          next unless mm_acceptable($m);
          push @matrices, $m;
        }
      }
    }
  }
  print "matrices ",scalar(@matrices),"\n";

  foreach my $m (@matrices) {
    if (mm_non_integer($m)) {
      print "non-integer ",mm_string($m),"\n";
    }
  }
  sub mm_non_integer {
    my ($m) = @_;
    return ! (is_integer($m->[0]) && is_integer($m->[1])
              && is_integer($m->[2]) && is_integer($m->[3]));
  }

  my $mm_count = 5;
  my @mm_indices = (0 .. $mm_count-1);
 MM_LIST: for (;;) {
    {
      my @mm_list = map {$matrices[$_]} @mm_indices;
      if (m3_coverage(@mm_list)) {
        print m3_string(@mm_list),"\n";
      }
    }
    my $pos = $mm_count-1;
    for (;;) {
      $mm_indices[$pos]++;
      if ($mm_indices[$pos] <= $#matrices - ($mm_count-1 - $pos)) {
        last;
      }
      $pos--;
      if ($pos < 0) {
        last MM_LIST;
      }
    }
    while (++$pos < $mm_count) {
      $mm_indices[$pos] = $mm_indices[$pos-1] + 1;
    }
  }

  sub mm_acceptable {
    my ($m) = @_;
    ### mm_acceptable(): mm_string($m).'  '.mm_name($m)
    foreach my $i (0 .. $#p) {
      my ($p,$q) = mm_descend($m, $p[$i],$q[$i]);
      return 0 unless pq_acceptable($p,$q);
    }
    return 1;
  }

  sub m3_coverage {
    my @mm_list = @_;
    ### m3_coverage(): m3_string(@mm_list)
    {
      my %seen;
      my @pending_p = (2);
      my @pending_q = (1);
      for (1 .. 6) {
        my @new_pending_p;
        my @new_pending_q;
        while (@pending_p) {
          my $p = pop @pending_p;
          my $q = pop @pending_q;
          if ($seen{"$p,$q"}++) {
            ### duplicate p,q ...
            return 0;
          }
          foreach my $m (@mm_list) {
            my ($new_p,$new_q) = mm_descend($m,$p,$q);
            push @new_pending_p, $new_p;
            push @new_pending_q, $new_q;
          }
        }
        @pending_p = @new_pending_p;
        @pending_q = @new_pending_q;
      }
      # foreach my $pq (@pq) {
      #   if (! $seen{$pq}) {
      #     return 0;
      #   }
      # }
    }
    if (1) {
    PQ: foreach my $i (1 .. $#p) {
        my $p = $p[$i];
        my $q = $q[$i];
        foreach my $m (@mm_list) {
          if (mm_upward($m, $p,$q)) { next PQ; }
        }
        ### no upward for p,q: "p=$p q=$q"
        return 0;
      }
    }
    return 1;
  }

  sub mm_descend {
    my ($m, $p,$q) = @_;
    ### assert: @_ == 3
    ### assert: @$m == 4
    ### assert: defined $m->[0]
    ### assert: defined $m->[1]
    ### assert: defined $m->[2]
    ### assert: defined $m->[3]
    ### assert: defined $p
    ### assert: defined $q
    return ($m->[0]*$p + $m->[1]*$q,
            $m->[2]*$p + $m->[3]*$q);
  }
  # (a b)(p)  inverse (d -b)
  # (c d)(q)          (-c a) / det
  # det = ad-bc
  sub mm_upward {
    my ($m, $p,$q) = @_;
    my $det = mm_determinant($m);
    my $ap = $m->[3]*$p - $m->[1]*$q;
    my $aq = - $m->[2]*$p + $m->[0]*$q;
    if (! is_integer($ap)) { return 0; }
    if (! is_integer($aq)) { return 0; }
    if ($ap % $det || $aq % $det) { return 0; }
    $ap /= $det;
    $aq /= $det;
    if (! pq_acceptable($ap,$aq)) {
      return 0;
    }
    return 1;
  }
  sub mm_determinant {
    my ($m) = @_;
    return ($m->[0] * $m->[3] - $m->[1] * $m->[2]);
  }
  sub m3_string {
    my (@mm_list) = @_;
    return join('  ',map {mm_string($_)} @mm_list)
      .'  '.join('',map {mm_name($_)} @mm_list);
  }
  sub mm_string {
    my ($m) = @_;
    return $m->[0].','.$m->[1].','.$m->[2].','.$m->[3];
  }
  BEGIN {
    my %mm_name = ('2,-1,1,0' => "U",
                   '2,1,1,0' => "A",
                   '1,2,0,1' => "D",
                   '1,1,0,2' => "K1",
                   '2,0,1,-1' => "K2",
                   '2,0,1,1' => "K3");
    sub mm_name {
      my ($m) = @_;
      return $mm_name{join (',',@$m)} || '.';
    }
  }
  exit 0;
}

{
  # OEIS grep for seq of matrix applied repeatedly
  require Math::BigInt;

  my $coordinates_choices = Math::PlanePath::PythagoreanTree
    ->parameter_info_hash->{'coordinates'}->{'choices'};
  my $zero = Math::BigInt->new(0);

  require Math::NumSeq::PlanePathN;
  # Math::NumSeq::PlanePathN::_bigint()->new(0);

  my @nlists;
  foreach my $digit (0 .. 2) {
    foreach my $depth (1 .. 10) {
      push @{$nlists[$digit]}, ternarytree_join_lowtohigh([($digit) x $depth], $zero);
    }
    print join(',', @{$nlists[$digit]}),"\n";
  }

  foreach my $tree_type ('UAD', 'FB', 'UMT') {
    foreach my $matrix ($tree_type eq 'UMT' ? 2 : 0 .. 2) {

      my %seen_coord_name;
      foreach my $coordinates (@$coordinates_choices) {
        my $path = Math::PlanePath::PythagoreanTree->new(tree_type => $tree_type,
                                                         coordinates => $coordinates);
        foreach my $coord (1, 2, 3) {
          my $coord_name = ($coord & 1 ? substr($coordinates,0,1) : '')
            . ($coord & 2 ? substr($coordinates,1,1) : '');
          next if $seen_coord_name{$coord_name}++;

          my @values;
          foreach my $n (@{$nlists[$matrix]}) {
            my @xy = $path->n_to_xy($n);
            if ($coord & 1) { push @values, $xy[0]; }
            if ($coord & 2) { push @values, $xy[1]; }
            ### @xy
          }
          require Math::OEIS::Grep;
          Math::OEIS::Grep->search(array => \@values,
                                   name => "$tree_type matrix=$matrix coord=$coord_name ($coord of $coordinates)");
        }
      }
    }
  }
  exit 0;

  sub ternarytree_join_lowtohigh {
    my ($aref, $zero) = @_;
    return ((3+$zero)**scalar(@$aref) + 1)/2    # tree_depth_to_n()
      + digit_join_lowtohigh($aref,3,$zero);  # digits within this depth
  }
}


{
  # upper parallelogram
  require Math::Geometry::Planar;
  my $root = Math::Matrix->new([2],[1]);
  my $start_l = U_2x2() * $root;
  my $start_r = A_2x2() * $root;
  ### $start_l
  ### $start_r

  my $vtop = $root;
  my $vbot = $root;
  foreach my $i (0 .. 5) {
    ### $vtop
    ### $vbot
    my $v_l = U_2x2() * $vbot;
    my $v_r = A_2x2() * $vbot;

    my $planar = Math::Geometry::Planar->new;
    my $points = [[mvec_to_pq($start_l)],
                  [mvec_to_pq($v_l)],
                  [mvec_to_pq($v_r)],
                  [mvec_to_pq($start_r)]];
    ### $points
    $planar->points($points);
    my $a = $planar->area;
    print "$i  $a\n";

    $vtop = U_2x2() * $vtop;
    $vbot = D_2x2() * $vbot;
  }
  exit 0;
}

{
  # in-between blocks

  require Math::Geometry::Planar;
  my $planar = Math::Geometry::Planar->new;
  my $t = 0;
  $planar->points([[2,1], [3,2], [5,2], [4,1]]); # initial parallelogram
  my $a = $planar->area;
  say $a;
  $t += $a;
  $planar->points([[3,2], [5,2], [13,6], [11,6]]); # upper in-between
  $a = $planar->area;
  say $a;
  $t += $a;
  $planar->points([[4,1], [5,2], [11,4], [10,3]]); # lower in-between
  $a = $planar->area;
  say $a;
  $t += $a;
  say $t;
  say 3*14+$t;
  exit;
}

{
  # row area recurrence

  # a0 = 0
  # a1 = 3*a0 + 6*1-4 = 2
  # a2 = 3*a1 + 6*2-4 = 14
  #    = 3*(3*a0 + 6*1-4) + 6*2-4

  # a[i] = 3*a[i-1] + 6*i-4
  #      = 3*(3*a[i-2] + 6*(i-1)-4) + 6*i-4
  #      = 3*3*a[i-2] + 3*(6*(i-1)-4) + 6*i-4
  #      = 3*3*3*a[i-3] + + 3*3*(6*(i-2)-4) + 3*(6*(i-1)-4) + 6*i-4

  # 3*3*4 + 3*4 + 4 = 4*(3^i-1)/(3-1)
  #
  # 3*3*3*(i-3) + 3*3*(i-2) + 3*(i-1) + i = i*(3^i-1)/(3-1) - ...
  #
  # (3*3*3*3)*4 + (3*3*3)*3 + (3*3)*2 + (3)*1 + (1)*0  k=5
  # = 3*(3^4-1)/2 + 3^2*(3^3-1)/2 + 3^3*(3^2-1)/2 + 3^4*(3^1-1)/2
  # = (3^5 - 3 + 3^5 - 3^2 + 3^5 - 3^3 + 3^5 - 3^4)/2
  # = 4*3^5/2 - (3 + 3^2 + 3^3 + 3^4)/2
  # = 4*3^5/2 - (1 + 3^1 + 3^2 + 3^3)*3/2
  # = 4*3^5/2 - ((3^4-1)/2)*3/2

  #  + ...
  # =  (3^i -     1)/2
  #  + (3^i -   3*1)/2
  #  + (3^i - 3*3*1)/2
  #  + ...
  # =  [ (i+1)*3^i  - (3^(i+1)-1)/2 ]/2
  #
  # a[i] = (6i-4)*(3^(i+1)-1)/2 - 6*(i*3^i  - (3^(i+1)-1)/2)
  #      = (6i-4)*(3^(i+1)-1)/2 - 6*i*3^i  - 6*(3^(i+1)-1)/2
  #      = (6i-4-6)*(3^(i+1)-1)/2 - 6*i*3^(i+1)
  #      = (6i-10)/2 * 3^(i+1) - (6i-10)/2 - 6*i*3^(i+1)
  #      = (3i-5 - 6i) * 3^(i+1) - (6i-10)/2

  foreach my $depth (1 .. 10) {
    my $want = uad_area_by_points($depth);
    my $got = uad_area_by_formula($depth);
    print "$depth $want $got\n";
  }

  #            3^k - 1
  # area = 5 * ------- - 3k = 5*3^(k-1)-3  + ... + 5*3^0-3
  #               2
  sub uad_area_by_formula {
    my ($k) = @_;
    my $total = 0;
    foreach my $i (0 .. $k-1) {
      $total += $i * 3**$i;
    }
    $total = (3**$k-1)/2 * (6*$k-4) - 6*((2*$k-3)*3**$k + 3)/4;
    $total = (3**$k-1) * (3*$k-2) - (3/2*(2*$k-3)*3**$k + 3/2*3);
    $total = (3**$k-1) * (3*$k-2) - (1/2*(6*$k-9)*3**$k + 9/2);
    $total = (3*$k-2)*3**$k - (3*$k-2) - (3*$k-9/2)*3**$k - 9/2;
    $total = (3*$k-2 - (3*$k-9/2))*3**$k - 3*$k + 2 - 9/2;

    $total = 0;
    foreach my $i (0 .. $k-1) {
      $total += 5* 3**$i - 3;
    }
    # $total -= 3*$k;

    $total = 5*(3**$k-1)/2 - 3*$k;
    return $total;
  }
  sub uad_area_by_recurrence {
    my ($depth) = @_;
    if ($depth <= 0) {
      return 0;
    }
    return uad_area_by_recurrence($depth-1)*3 + 6*$depth - 4;
  }
  sub uad_area_by_points {
    my ($depth) = @_;

    my @points = ([2,1]);
    my $path = Math::PlanePath::PythagoreanTree->new(coordinates => 'PQ',
                                                     digit_order => 'LtoH');
    my $n_depth = $path->tree_depth_to_n($depth);
    foreach my $offset (0 .. 3**$depth-1) {
      my $n = $n_depth + ternary_1s_reverse($offset);
      my ($x,$y) = $path->n_to_xy($n);
      push @points, [$x,$y];
    }
    require Math::Geometry::Planar;
    my $planar = Math::Geometry::Planar->new;
    $planar->points(\@points);
    return $planar->area;
  }
  exit 0;
}

{
  # row area SVG image

  my $depth = 2;
  my $yfactor = 2;
  my $path = Math::PlanePath::PythagoreanTree->new(coordinates => 'PQ',
                                                   digit_order => 'LtoH');

  my $depth_to_points_list = sub {
    my ($depth) = @_;
    my @points = (Math::Matrix->new([2],[1]));
    my $n_depth = $path->tree_depth_to_n($depth);
    foreach my $offset (0 .. 3**$depth-1) {
      my $n = $n_depth + ternary_1s_reverse($offset);
      my ($x,$y) = $path->n_to_xy($n);
      push @points, Math::Matrix->new([$x],[$y]);
    }
    return @points;
  };
  my $I_2x2 = Math::Matrix->new([1,0],[0,1]);

  my $scale;
  {
    my $max = 0;
    foreach my $n ($path->tree_depth_to_n($depth+1) .. $path->tree_depth_to_n_end($depth+1)) {
      my ($x,$y) = $path->n_to_xy($n);
      $max = max($max, $x, $y*$yfactor);
    }
    $scale = 800 / $max * .95;
  }
  print "scale $scale\n";
  my $xscale = $scale;
  my $yscale = $scale * -$yfactor;

  eval "use SVG; 1" or die;
  my $svg = SVG->new(width => 800, height => 800);
  $svg->rectangle(x=>0, y=>0, width=>800, height=>800,
                  style => {'fill'           => 'black',
                            'stroke'         => 'black',
                            'stroke-opacity' =>  .5,
                            'fill-opacity'   =>  1 });

  my $group = $svg->group(transform =>
                          "translate(0,800) scale($xscale,$yscale)");
  my $draw = sub {
    my ($depth, $m, $colour) = @_;
    my @points = map {$m * $_} $depth_to_points_list->($depth);
    ### @points
    foreach my $p (@points) {
      my ($p,$q) = mvec_to_pq($p);
      $group->circle(cx=>$p,cy=>$q, r=>.1,
                     style => {'stroke-width' => 0,
                               fill => $colour,
                              });
    }
    my $path = mvecs_to_svg_path(@points);
    $group->polygon(%$path,
                    style => {'stroke'         => $colour,
                              'stroke-opacity' => .5,
                              fill => 'none',
                              # 'fill'           => $colour,
                              'stroke-width'   => .1,
                              'stroke-linejoin' => 'miter',
                             });
  };

  # $draw->($depth, $I_2x2, 'white');
  $draw->($depth, U_2x2(), 'pink');
  $draw->($depth, A_2x2(), 'cyan');
  $draw->($depth, D_2x2(), 'lime');

# # ->multiply_scalar(3)
#   $draw->($depth-1, $I_2x2, 'red');
#   $draw->($depth,  $I_2x2, 'white');

  my $xml = $svg->xmlify;
  print "$xml\n";
  ### $xml
  require IPC::Run;
  IPC::Run::run (['see','image/svg+xml:-'], '<', \$xml);
  exit 0;

  sub mvecs_to_svg_path {
    my @points = @_;
    my (@x, @y);
    foreach my $v (@_) {
      my ($p,$q) = mvec_to_pq($v);
      push @x, $p;
      push @y, $q;
    }
    return $svg->get_path(x=>\@x, y=>\@y, -type=>'polygon');
  }
}

{
  # T repeatedly
  # T = (1 3)     p' = p + 3*q      p_i = 3.2^i - 1
  #     (0 2)     q' = 2*q          q_i = 2^i
  {
    my $pq = Math::Matrix->new([2],[1]);
    print "$pq->[0][0],$pq->[1][0]";
    foreach my $i (1 .. 5) {
      $pq = T_2x2() * $pq;
      print " \\to $pq->[0][0],$pq->[1][0]";
    }
    print "\n";
  }
  print "\n";
  {
    my $pq = Math::Matrix->new([2],[1]);
    foreach my $i (0 .. 4) {
      my $p = $pq->[0][0];
      my $q = $pq->[1][0];
      my ($a,$b) = pq_to_abc($p,$q);
      my $c = sqrt($a*$a + $b*$b);
      $p == 3*2**$i-1 or die;
      $q == 2**$i or die;
      printf "%d & %2d & %2d & \\texttt{%6b} & \\texttt{%5b} \\\\\n",
        $i, $p,$q, $p,$q;
      $pq = T_2x2() * $pq;
    }
  }

  # p[i] = 3*2^i-1
  # q[i] = 2^i
  # p-q = 3*2^i-1 - 2^i
  #     = 2^(i+1) - 1
  # B = 2pq = (3*2^i-1)*2^(i+1)
  # A = (3.2^i-1)^2 - 2^2i
  #   = 9.2^2i - 6.2^i + 1 - 2^2i
  #   = 8.2^2i - 6.2^i + 1                    36-4.8 = 4
  #   = (4.2^2i - 1)(2.2^2i - 1)
  # C = (3.2^i-1)^2 - 2^2i
  #   = 10.2^2i - 6.2^i + 1                    36-4.10 = -4
  # binomial(2^(i+2)-1, 2)
  #   = (2^(i+2)-1).(2^(i+2)-2) / (1.2)
  #   = (2^(i+2)-1).(2^(i+1)-1)
  #   = (2^(i+2)-1).(2^(i+1)-1)
  #   = (4.2^i-1).(2.2^i-1)
  # C-A = p^2+q^2 - (p^2-q^2)
  #     = 2q^2
  # C-B = p^2+q^2 - 2pq
  #     = (p-q)^2

  {
    my $pq = Math::Matrix->new([2],[1]);
    foreach my $i (0 .. 4) {
      my $p = $pq->[0][0];
      my $q = $pq->[1][0];
      my ($a,$b) = pq_to_abc($p,$q);
      my $c = sqrt($a*$a + $b*$b);
      $a == (4*2**$i - 1) * (2*2**$i - 1) or die;
      $a == binomial(2**($i+2)-1, 2) or die;
      $b == (3*2**$i-1)*2**($i+1) or die;
      my $diff = $c - $a;
      $diff == 2**(2*$i+1) or die;
      printf "%d & %4d & %4d & %4d &\n   \\texttt{%11b} & \\texttt{%11b} & \\texttt{%12b} \\\\\n",
        $i, $a,$b,$c, $a,$b,$c;
      $pq = T_2x2() * $pq;
    }
  }
  exit 0;

  sub pq_to_abc {
    my ($p,$q) = @_;
    return ($p*$p - $q*$q,
            2*$p*$q,
            $p*$p + $q*$q);
  }
  sub binomial {
    my ($n, $k) = @_;
    require Math::BigInt;
    return Math::BigInt->new($n)->bnok($k);
  }
}
{
  require Math::Geometry::Planar;
  my $planar = Math::Geometry::Planar->new;
  $planar->points([[2,1], [7,2], [9,2], [6,1]]);   # 0,7,8,9 area=3
  print $planar->area,"\n";
  exit 0;
}

{
  # Barning
  print U_3x3->invert() * A_3x3,"\n";
  print D_3x3->invert() * A_3x3,"\n";

  print U_2x2->invert() * A_2x2,"\n";
  print D_2x2->invert() * A_2x2,"\n";
  exit 0;
}

{
  # all primitive triples with C-A = 2^k
  require Math::PlanePath::GcdRationals;
  foreach my $a (3 .. 500) {
    next unless $a & 1;
    foreach my $b (3 .. 500) {
      my $cc = $a*$a + $b*$b;
      my $c = int(sqrt($cc));
      next unless $c*$c == $cc;
      my $diff = $c-$a;
      next unless $diff >= 2;
      next unless is_pow2($diff);
      my $log = log2($diff);
      next unless $log & 1;
      next unless Math::PlanePath::GcdRationals::_gcd($a,$b) == 1;
      print "$a $b $c   $diff\n";
    }
  }
  exit 0;

  sub log2 {
    my ($n) = @_;
    unless ($n >= 1) {
      return 0;
    }
    my $count = 0;
    until ($n <= 1) {
      $n >>= 1;
      $count++;
    }
    return $count;
  }
}

{
  # cases how to go 2,1 to 3,2 etc points

  # ad-bc = ad+bd - bd-bc
  #       = (a+b)d - b(c+d)
  #       <= (a+b)*(0-c) + (c+d)*(1-a)
  #       = -ac+bc + c -ac +d -ad
  #
  # [4,-3;1,0]*[3;2] - [6;3]
  # ap+bq, p
  # p=b+1, q=b     a(b+1)+b.b, b+1
  # p=b,   q=1     ab+b, b              ap+b, b   for b even
  # p=3b,  q=2     3ab+b, 3b            ap+b, b   for b odd
  # p=k+1, q=k     a(k+1)+bk, k+1
  # in general d=0 must have b=+/-1 only

  my $term_min = -20;
  my $term_max = 20;
  my @term_list = ($term_min .. $term_max);
  my $count_fail = 0;
  my $count_total = 0;

  foreach my $elem ([2,1, 3,2],
                    [2,1, 4,1],
                    [2,1, 4,3],
                    [2,1, 5,2],

                    [3,2, 4,1],
                    [3,2, 4,3],
                    [3,2, 5,2],

                    [4,1, 5,2],
                    [4,3, 5,2],

                    # [5,4, 6,3],
                    # [6,5, 7,4],
                    # [7,6, 8,5],
                    #
                    # [2,1, 5,4],
                    # [3,2, 5,4],
                    # [4,1, 5,4],
                    # [4,3, 5,4],
                   ) {
    my ($p,$q, $to_p,$to_q) = @$elem;
    print "$p,$q to $to_p,$to_q\n";

    foreach my $a (@term_list) {
      foreach my $b (@term_list) {
        my $p_got = $a*$p + $b*$q;
        next unless $p_got == $to_p;
        # print " a=$a b=$b  for $p_got\n";

        foreach my $c (@term_list) {
          foreach my $d (@term_list) {
            my $q_got = $c*$p + $d*$q;
            next unless $q_got == $to_q;
            # print "  c=$c d=$d  for $q_got\n";

            my $fail = abcd_conditions_fail($a,$b,$c,$d);
            next if $fail =~ /^\*/;

            my $str;
            if ($fail) {
              $str = "fails $fail";
              $count_fail++;
            } else {
              $str = "matrix ".matrix_name($a,$b,$c,$d);
            }
            printf "  %2d %2d %2d   %2d %2d %2d   %s\n",
              $a,$a+$b,$b, $c,$c+$d,$d, $str;
            $count_total++;
          }
        }
      }
    }
  }
  print "count fail $count_fail\n";
  print "count total $count_total\n";

  # Return string describing how matrix $a,$b,$c,$d fails lemma.
  # Return empty string if it doesn't fail.
  sub abcd_conditions_fail {
    my ($a,$b,$c,$d) = @_;
    unless ($a >= 1) { return "*a>=1" }
    unless ($c >= 0) { return "*c>=0" }
    unless ($a > $c) { return "*a>c" }
    unless ($a+$b >= 1) { return "*a+b>=1" }
    unless ($c+$d >= 0) { return "*c+d>=0" }
    unless ($a+$b >= $c+$d) { return "a+b >= c+d" }
    unless (($a+$c)&1) { return "a+c=1 mod 2" }
    unless (($b+$d)&1) { return "b+d=1 mod 2" }
    if ($a==1 && $b==0) { return "a=1,b=0 not allowed" }
    my $det = $a*$d - $b*$c;
    unless (is_pow2(abs($det))) { return "det=2^r"; }

    if ($c==0 && $d==0) { return "c=0,d=0 not allowed" }
    if ($c==0) {
      unless ($a==1) { return "if c=0 then a=1" }
    }
    if ($d == 0) {
      unless ($b==1 || $b==-1) { return "if d=0 then b=+/-1" }
    }
    return '';
  }
  exit 0;
}

{
  # factorizing matrices

  require Math::PlanePath::GcdRationals;
  my %gcd;
  my $gcd_func = sub {
    my ($x,$y) = @_;
    $x = abs($x);
    $y = abs($y);
    return ($gcd{$x}->{$y} //= Math::PlanePath::GcdRationals::_gcd($x,$y));
  };
  # $gcd_func = \&Math::PlanePath::GcdRationals::_gcd;

  my @matrices;
  my %primes;
  my $limit = 33;

  foreach my $a (1 .. $limit) {          # a >= 1
    foreach my $b (1-$a .. $limit) {     # a+b >= 1    so b >= 1-a
      next if $a == 1 && $b == 0;    # not a=1,b=0

      foreach my $c (0 .. $a-1) {    # a > c >= 0
        if ($c == 0) {
          next unless $a == 1;       # if c=0 then a=1 only
        }
        next unless ($a+$c) & 1;
        next unless $gcd_func->($a,$c) == 1;    # gcd(a,c)=1

        foreach my $d (-$c .. $limit) {  # c+d >= 0 so d >= -c
          if ($d == 0) {
            next unless $b == 1 || $b == -1;   # if d=0 then b=+/-1 only
          }
          next unless $a+$b >= $c+$d;          # a+b >= c+d
          next unless ($b+$d) & 1;
          next unless $gcd_func->($b,$d) == 1; # gcd(b,d)=1

          my $det = $a*$d - $b*$c;
          next if $det == 0;
          my $abs_det = abs($det);

          my $abs_det_sans_2s = odd_part($abs_det);
          next unless $abs_det_sans_2s == 1;

          push @matrices, [$a,$b,$c,$d];
          $primes{"$a,$b,$c,$d"} = 1;
        }
      }
    }
  }
  print scalar(@matrices)," matrices\n";

  foreach my $x (@matrices) {
    foreach my $y (@matrices) {
      my $a = $x->[0] * $y->[0] + $x->[1] * $y->[2];
      my $b = $x->[0] * $y->[1] + $x->[1] * $y->[3];
      my $c = $x->[2] * $y->[0] + $x->[3] * $y->[2];
      my $d = $x->[2] * $y->[1] + $x->[3] * $y->[3];
      delete $primes{"$a,$b,$c,$d"};
    }
  }
  my @primes = keys %primes;
  print "primes ",scalar(@primes),"\n";

  foreach my $m (sort keys %primes) {
    my ($a,$b,$c,$d) = split /,/, $m;
    my $det = $a*$d - $b*$c;
    print "$m  det=$det\n";
    if (! abcd_test_pq_list($a,$b,$c,$d)) {
      die;
    }
  }
  exit 0;

  # sub math_matrix_divisible {
  #   my ($m, $d) = @_;
  #   my $mdet = abs($m->determinant);
  #   my $ddet = abs($d->determinant);
  #   if ($mdet % $ddet != 0) { return 0; }
  #   $d->invert;
  #   return 1;
  # }

  sub odd_part {
    my ($n) = @_;
    while ($n > 0 && ($n & 1) == 0) {
      $n >>= 1;
    }
    return $n;
  }
}
{
  # x',y' determining p',q' GCD

  # xp+yq = 1
  # x(dp'-bq')/det + y(-cp'+aq)/det = 1
  # (xd-yc)p' + (-xb+ya)q' = det
  # which is a pair of integers
  #   x' =  dx - cy
  #   y' = -bx + ay
  # so gcd(p',q') is a divisor of det
  # but p' and q' opposite parity, so their gcd is odd and so must be 1



  # xp+yq = 1
  # ?E+OO = O   p even, q odd   has y odd
  # OO+?E = O   p odd, q even   has x odd

  # y = (1-xp)/q
  # (x y)(p) = 1
  #      (q)
  #
  # (x y)(d -b)/det (a b)(p) = 1
  #      (-c a)     (c d)(q)
  #
  # (x y)(d -b)/det (p') = 1
  #      (-c a)     (q')
  #
  # (xd -cy, -bx + ay)/det (p') = 1
  #                        (q')
  #
  # (x y)(d -b) (p') = det
  #      (-c a) (q')
  # x' =  dx - cy
  # y' = -bx + ay
  #
  # x'p' + y'q' = det     so gcd(p',q') divisor of det

  # 2.1 + 1.-1 = 1  x=1,y=-1
  # (6 2) det=4
  # (1 1)
  # * 2,1
  #   p'=12-2=10      10*1 - 3*3 = 1 x'=1 y'=3
  #   q' = 2+1=3
  # x' = (1.1 - 1.-1)/4 = 2/4
  # y' = (-2x+6y)/4 = (-2-6)/4 = -2
  # x'*p' + y'*q'
  # = 2/4.10 + -2.3
  # = 5 + -2.3 = -1
  # some 2s from p' some from x'
  # p' = something.2^alpha
  # x' =  dx - cy


  # ad - bc = det
  # EE   OO = O so +/-1    a,b opposite, x',y' integers
  # EO   EO = E so +/-2^r  \ a,b both even
  # OE   OE = E so +/-2^r  / a,b both odd
  #
  # a,b both even pow=2^alpha a=A.2^alpha b=B.2^alpha
  # c,d both odd
  # A.2^alpha.d - B.2^alpha.c = 2^r
  # A.d - B.c = 2^(r-alpha)
  # p' = ap+bq = something.2^alpha
  # q' = cp+dq = O.? + O.opp == p+q = odd
  # y' = -bx + ay = something.2^alpha
  #    = -bx + (1-xp)/q
  #    = (bqx + 1 - xp)/q
  #    = (x(bq -p) + 1)/q
  #
  # x = x0 + kq
  # y = y0 - kp
  # x' = d(x0+kq) - c(y0 - kp)
  #    = dx0 - cy0 + dkq + ckp
  #    = dx0 - cy0 + k(cp + dq)       q'=cp+dq
  #    = dx0 - cy0 + k.q'
  # y' = -b(x0 + kq) + a(y0 - kp)
  #    = -bx0 - bkq  + ay0 - akp
  #    = -bx0 + ay0 - k(ap + bq)
  #    = -bx0 + ay0 - kp'             p'=ap+bq


  # gcd(ap+bq, cp+dq)
  # = gcd( (a-c)p+(b-d)q, cp+dq)

  # xp+yq=1
  # (x+kq)p + (y+kp)q = 1
  #


  my $a = 10;
  my $b = 2;
  my $c = 1;
  my $d = 1;
  my $det = $a*$d - $b*$c;
  print "$a,$b, $c,$d  det=$det\n";
  abcd_test_pq_list($a,$b,$c,$d)
    or die;
  if (abcd_conditions_fail($a,$b,$c,$d)) {
    die;
  }

  my @p_list;
  my @q_list;
  foreach my $p (2 .. 2) {
    foreach my $q (1 .. $p-1) {
      if (pq_acceptable($p,$q)) {
        push @p_list, $p;
        push @q_list, $q;
      }
    }
  }
  print "pq list ",scalar(@p_list),"\n";

  # (x y) ( d -b)     x' =  xd - yc
  #       (-c  a)     y' = -xb + ya
  #
  foreach my $i (0 .. $#p_list) {
    my $p = $p_list[$i];
    my $q = $q_list[$i];
    my ($x,$y) = pq_find_xy_egcd($p,$q);
    my $p_parity = parity_str($p);
    my $q_parity = parity_str($q);
    print "p=$p  $p_parity\n";
    print "q=$q  $q_parity\n";
    print "  x=$x  odd since p even\n";
    print "  y=$y  any parity\n";
    my $g = $x*$p+$y*$q;
    print "  xp+yq = $x*$p+$y*$q = $g\n";

    my $pp = $a*$p + $b*$q;
    my $qq = $c*$p + $d*$q;
    my $xx =  $x*$d - $y*$c;
    my $yy = -$x*$b + $y*$a;
    print "  pp = $a*$p + $b*$q = $pp\n";
    print "  qq =  $c*$p + $d*$q = $qq\n";
    print "  xx =  xd-yc =  $x*$d - $y*$c = $xx\n";
    print "  yy = -xb+ya = -$x*$b + $y*$a = $yy\n";
    my $cross = $xx*$pp + $yy*$qq;
    print "  xx*pp + yy*qq = $cross\n";
    print "  $xx*$pp + $yy*$qq = $cross\n";
    my $xxpp = $xx*$pp;
    my $yyqq = $yy*$qq;
    print "   $xxpp + $yyqq\n";
    my $xxpp_pow2 = even_part(abs($xxpp));
    my $yyqq_pow2 = even_part(abs($yyqq));
    print "    $xxpp_pow2       $yyqq_pow2\n";
  }

  exit 0;

  sub pq_find_xy_egcd {
    my ($p, $q) = @_;

    # seek xp+yq=1
    my $x = 1;
    my $y = 0;
    for (;;) {
      my $g = $x*$p + $y*$q;
      if ($g == 1) {
        return ($x,$y);
      }
      if ($g == 0) {
        die "oops, p,q common factor";
      }
      if ($g < 0) {
        $x++;
        $y = 0;
        if ($x > $q) {
          die "oops, xp+yq=1 not found";
        }
      } else {
        $y--;
      }
    }
  }

  sub parity_str {
    my ($n) = @_;
    return ($n & 1 ? "odd" : "even");
  }

  sub even_part {
    my ($n) = @_;
    die "oops even_part($n)" if $n < 1;
    my $ret = 1;
    until ($n & 1) {
      $ret <<= 1;
      $n >>= 1;
    }
    return $ret;
  }
}


{
  # finding (det,det)

  # (d  -b)(x) = (s)     dx-by = s
  # (-c  a)(y)   (1)     ay-xc=1

  # Now take the case a!=b mod 2.  Consider the identity
  #     (a b)(p) = (s.det)    where s even
  #     (c d)(q)   (  det)
  #     (a b) (d -b) (s) = (s.det)
  #     (c d)(-c  a) (t)   (t.det)

  #     (a b)( xd - yb) = (x.det)
  #     (c d)(-xc + ya)   (y.det)
  # gcd(a,c)=1 so -xc+ya=1 has a set of solutions
  #   x = x0 + ka
  #   y = y0 + kc   any integer k
  # all of which give q=1
  # p = (x0+ka)d - (y0+kc)*b
  #   = x0*d-y0*b + (ad-bc)*k
  # so p increments or decrements with k, choose k big enough positive or
  # negative to make p >= 2.
  # Then ad-bc=odd so choose k odd or even the same as x0*d-y0*b is odd or
  # even, thereby giving p even.
  # The result is a pair (p,1) for which
  #     (a b)(p) = (x.det)
  #     (c d)(1)   (y.det)
  # which has common factor det in the child p',q'.  Must have det=+/-1.
  #
  # ap+b = x.(ad-bc)
  # cp+d = y.(ad-bc)
  # p = x0*d-y0*b + (ad-bc)*k
  #
  # gcd(a,c)=1 so exists x,y with -xc+ya=1
  # consider p=xd-yb + (ad-bc)k
  #
  # where k is chosen same parity as xd-yb, so that p is even, and k big
  # enough positive or negative to make p>=2.  The child of pair p,1 is
  #
  # p' = a(x*d-y*b + (ad-bc)*k) + b
  #    = x.a.d - y.a.b + a.(ad-bc)k + b
  #    = x.a.d - (1+x.c).b + a.(ad-bc)k + b     since  y.a=1+x.c
  #    = x.a.d - (b+x.c.b) + a.(ad-bc)k + b
  #    = x.a.d - b - x.c.b + a.(ad-bc)k + b
  #    = x.a.d - x.c.b + a.(ad-bc)k
  #    = x(a.d - b.c) + a.(ad-bc)k
  # q' = c(x*d-y*b + (ad-bc)*k) + d
  #    = x.d.c - y.b.c + c.(ad-bc)*k + d
  #    = (y.a-1).d - y.b.c + c.(ad-bc)*k + d        since x.c = y.a-1
  #    = y.a.d -d - y.b.c + c.(ad-bc)*k + d
  #    = y.a.d - y.b.c + c.(ad-bc)*k
  #    = y(a.d - b.c) + c.(ad-bc)*k
  # This p',q' has common factor det=ad-bc, so must have det=+/-1.


  require Math::PlanePath::GcdRationals;
  my %gcd;
  my $gcd_func = sub {
    my ($x,$y) = @_;
    $x = abs($x);
    $y = abs($y);
    return ($gcd{$x}->{$y} //= Math::PlanePath::GcdRationals::_gcd($x,$y));
  };
  # $gcd_func = \&Math::PlanePath::GcdRationals::_gcd;

  foreach my $a (1 .. 10) {          # a >= 1
    foreach my $b (1-$a .. 20) {     # a+b >= 1    so b >= 1-a
      next if $a == 1 && $b == 0;    # not a=1,b=0
      my $gcd_ab = $gcd_func->($a,$b);

      # next unless ($a + $b) & 1;     # sub-cases a,b opposite
      next if ($a + $b) & 1;     # sub-cases a,b same

      foreach my $c (0 .. $a-1) {    # a > c >= 0
        if ($c == 0) {
          next unless $a == 1;       # if c=0 then a=1 only
        }
        next unless ($a+$c) & 1;
        next unless $gcd_func->($a,$c) == 1;    # gcd(a,c)=1
        my $gcd_abc = $gcd_func->($gcd_ab, $c);

        foreach my $d (-$c .. 10) {  # c+d >= 0 so d >= -c
          if ($d == 0) {
            next unless $b == 1 || $b == -1;   # if d=0 then b=+/-1 only
          }
          next if $c == 0 && $d == 0;          # not c=0,d=0
          next unless $a+$b >= $c+$d;          # a+b >= c+d
          next unless ($b+$d) & 1;
          next unless $gcd_func->($b,$d) == 1; # gcd(b,d)=1

          my $gcd_abcd = $gcd_func->($gcd_abc, $d);
          next unless $gcd_abcd == 1;

          # next unless $gcd_func->($a,$b) == 1;
          # next unless $gcd_func->($c,$d) == 1;
          # next unless $gcd_func->($a+$b,$c+$d) == 1;
          # next unless $gcd_func->($a+$c,$b+$d) == 1;

          my $det = $a*$d - $b*$c;
          next if $det == 0;
          my $abs_det = abs($det);

          print "$a,$b,$c,$d  det=$det\n";

          # seek ay-xc=1
          my $y = 1;
          my $x = 0;
          for (;;) {
            my $ayxc = $a*$y - $x*$c;
            if ($ayxc == 1) {
              last;
            }
            $x++;
            if ($x > $a) {
              $y++;
              $x = 0;
              if ($y > $c) {
                die "oops, ay-xc=1 not found";
              }
            }
          }

          # d(x+a)-b(y+c) - (dx-by)
          # = dx+da - by-bc - dx + by
          # = da-bc = det
          for my $i (1 .. 15) {
            my $ayxc = $a*$y - $x*$c;
            my $dxby = $d*$x - $b*$y;   # dx-by = s
            print "  x=$x,y=$y   ay-xc=$ayxc    dx-by=$dxby\n";
            if ($det >= 0) {
              $x += $a;
              $y += $c;
            } else {
              $x -= $a;
              $y -= $c;
            }
            last if $dxby >= 2 && $i >= 2;
          }
        }
      }
    }
  }
  exit 0;

  # return true if $n is a power-of-2
  sub is_pow2 {
    my ($n) = @_;
    unless ($n >= 1) {
      return 0;
    }
    until ($n & 1) { $n >>= 1; }
    return $n == 1;
  }
}

{
  # GCDs vs determinants
  #
  # if gcd(pp,qq)=1 whenever gcd(p,q)=1 then
  # det = +/-1          if a,b opposite parity
  # det = +/-2,4,8,etc  if a,b same parity

  # if det != 2^k then can find a gcd dividing det ...

  # ap+bq - cp - dq
  # = (a-c)p + (b-d)q
  # >= p + (b-d)q

  require Math::PlanePath::GcdRationals;
  my %gcd;
  my $gcd_func = sub {
    my ($x,$y) = @_;
    $x = abs($x);
    $y = abs($y);
    return ($gcd{$x}->{$y} //= Math::PlanePath::GcdRationals::_gcd($x,$y));
  };
  # $gcd_func = \&Math::PlanePath::GcdRationals::_gcd;


  my @p_list;
  my @q_list;
  foreach my $p (2 .. 500) {
    foreach my $q (1 .. $p-1) {
      if (pq_acceptable($p,$q)) {
        push @p_list, $p;
        push @q_list, $q;
      }
    }
  }

  foreach my $a (1 .. 10) {          # a >= 1
    foreach my $b (1-$a .. 20) {     # a+b >= 1    so b >= 1-a
      next if $a == 1 && $b == 0;    # not a=1,b=0
      my $gcd_ab = $gcd_func->($a,$b);

      # next unless ($a + $b) & 1;     # sub-cases a,b opposite
      # next if ($a + $b) & 1;     # sub-cases a,b same

      foreach my $c (0 .. $a-1) {    # a > c >= 0
        if ($c == 0) {
          next unless $a == 1;       # if c=0 then a=1 only
        }
        next unless ($a+$c) & 1;
        next unless $gcd_func->($a,$c) == 1;    # gcd(a,c)=1
        my $gcd_abc = $gcd_func->($gcd_ab, $c);

      DVAL: foreach my $d (-$c .. 10) {  # c+d >= 0 so d >= -c
          if ($d == 0) {
            next unless $b == 1 || $b == -1;   # if d=0 then b=+/-1 only
          }
          next if $c == 0 && $d == 0;          # not c=0,d=0
          next unless $a+$b >= $c+$d;          # a+b >= c+d
          next unless ($b+$d) & 1;
          next unless $gcd_func->($b,$d) == 1; # gcd(b,d)=1

          my $gcd_abcd = $gcd_func->($gcd_abc, $d);
          next unless $gcd_abcd == 1;


          next unless $gcd_func->($a,$b) == 1;
          next unless $gcd_func->($c,$d) == 1;
          next unless $gcd_func->($a+$b,$c+$d) == 1;
          next unless $gcd_func->($a+$c,$b+$d) == 1;


          my $det = $a*$d - $b*$c;
          next if $det == 0;
          my $abs_det = abs($det);

          my $abs_det_sans_2s = $abs_det;
          while ($abs_det_sans_2s > 0 && ($abs_det_sans_2s & 1) == 0) {
            $abs_det_sans_2s >>= 1;
          }

          my $ab_parity;
          if (($a + $b) & 1) {
            next if $abs_det == 1;
            $ab_parity = "a,b opposite";
          } else {
            # next if $abs_det == 2;
            next if $abs_det_sans_2s == 1;  # det=2^k allowed
            $ab_parity = "a,b same";
          }

          foreach my $i (0 .. $#p_list) {
            my $p = $p_list[$i];
            my $q = $q_list[$i];

            my $pp = $a*$p + $b*$q;
            my $qq = $c*$p + $d*$q;
            if ($qq >= $pp) {
              die "oops, qq too big";
            }

            my $gg = $gcd_func->($pp,$qq);
            next if $gg == 1;

            next unless $gg == $abs_det_sans_2s;  # looking for gcd=det/2^k

            if ($det % $gg != 0) {
              die;
            }
            print "$a,$b,$c,$d * ($p, $q) = ($pp, $qq)  gcd=$gg   det=$det     $ab_parity\n";
            next DVAL;
          }
          print "$a,$b,$c,$d  det=$det  no gcd>1 found    $ab_parity\n";

        }
      }
    }
  }
  exit 0;
}

BEGIN {
  my @p_list;
  my @q_list;
  foreach my $p (2 .. 500) {
    foreach my $q (1 .. $p-1) {
      if (pq_acceptable($p,$q)) {
        push @p_list, $p;
        push @q_list, $q;
      }
    }
  }
  sub abcd_test_pq_list {
    my ($a,$b,$c,$d) = @_;

    foreach my $i (0 .. $#p_list) {
      my $p = $p_list[$i];
      my $q = $q_list[$i];

      if (! pq_acceptable($a*$p + $b*$q,
                          $c*$p + $d*$q)) {
        return 0;
      }
    }
    return 1;
  }
}


{
  # determinants
  foreach my $m (U_2x2(),
                 A_2x2(),
                 D_2x2(),
                 M1_2x2(),
                 M2_2x2(),
                 M3_2x2(),
                 T_2x2()) {
    my $det = $m->determinant;
    print matrix_name($m)," det = $det\n";
  }
  exit 0;
}



{
  # UAD gnuplot row

  my $path = Math::PlanePath::PythagoreanTree->new(coordinates => 'PQ',
                                                   digit_order => 'LtoH');
  open my $fh, '>', 'row.dat' or die;
  print $fh "# DO NOT EDIT\n" or die;
  foreach my $depth (3) {
    foreach my $n ($path->tree_depth_to_n($depth) .. $path->tree_depth_to_n_end($depth)) {
      my $n_depth = $path->tree_depth_to_n($depth);
      my $offset = $n - $n_depth;

      # mangle
      if (1) {
        $offset = ternary_1s_reverse($offset);
        $n = $n_depth + $offset;
      }

      my ($x,$y) = $path->n_to_xy($n);
      print $fh "$x $y\n" or die;
    }
    close $fh or die;
    require IPC::Run;
    IPC::Run::run(['gnuplot','row.gnuplot']);
  }
  exit 0;
}






{
  # CW descent divisions

  foreach my $m (Math::Matrix->new([2,1],[1,1]),
                 Math::Matrix->new([1,1],[0,1]),
                 Math::Matrix->new([1,0],[1,1]),
                 Math::Matrix->new([1,1],[1,2])) {
    my $m_det = $m->determinant;
    foreach my $d (U_2x2(),
                   A_2x2(),
                   D_2x2(),
                   M1_2x2(),
                   M2_2x2(),
                   M3_2x2(),
                   T_2x2()) {
      my $d_det = $d->determinant;
      next unless $m_det % $d_det == 0;

      my $dinv = $d->invert;
      my $q = $m * $dinv;
      # $q = $q->negative;
      print $q;
      print matrix_name($q->[0][0],$q->[0][1],$q->[1][0],$q->[1][1]),
        matrix_name($q->[0][0],$q->[1][0],$q->[0][1],$q->[1][1]),
          "\n";
      print "\n";
    }
  }
  exit 0;
}

{
  print U_2x2()->invert,"\n";
  print M2_2x2()->invert,"\n";
  print T_2x2()->invert,"\n";
  exit 0;
}

{
  my $x2 = Math::Matrix->new([3,-1],[2,-2]);
  say $x2 * Math::Matrix->new([2],[1]);
  say $x2 * Math::Matrix->new([3],[2]);
  say $x2 * Math::Matrix->new([4],[1]);
  say $x2 * Math::Matrix->new([4],[3]);
  say $x2 * Math::Matrix->new([5],[2]);
  say $x2 * Math::Matrix->new([5],[4]);
  say $x2 * Math::Matrix->new([6],[1]);
  say $x2 * Math::Matrix->new([6],[5]);
  say "----";
  say U_2x2() * M2_2x2() * Math::Matrix->new([2],[1]);
  say U_2x2() * M2_2x2() * Math::Matrix->new([3],[2]);
  say U_2x2() * M2_2x2() * Math::Matrix->new([4],[1]);
  say U_2x2() * M2_2x2() * Math::Matrix->new([4],[3]);
  say U_2x2() * M2_2x2() * Math::Matrix->new([5],[2]);
  say U_2x2() * M2_2x2() * Math::Matrix->new([5],[4]);
  say U_2x2() * M2_2x2() * Math::Matrix->new([6],[1]);
  say U_2x2() * M2_2x2() * Math::Matrix->new([6],[5]);
  say U_2x2() * M2_2x2() * Math::Matrix->new([7],[2]);
  say U_2x2() * M2_2x2() * Math::Matrix->new([7],[4]);
  say U_2x2() * M2_2x2() * Math::Matrix->new([7],[6]);

  my @v_list = (
                Math::Matrix->new([2],[1]),
                Math::Matrix->new([3],[2]),
                Math::Matrix->new([4],[1]),
                Math::Matrix->new([4],[3]),
                Math::Matrix->new([5],[2]),
                Math::Matrix->new([5],[4]),
                Math::Matrix->new([6],[1]),
                Math::Matrix->new([6],[5]),
                Math::Matrix->new([7],[2]),
                Math::Matrix->new([7],[4]),
                Math::Matrix->new([7],[6]),
                Math::Matrix->new([8],[1]),
                Math::Matrix->new([8],[3]),
                Math::Matrix->new([8],[5]),
                Math::Matrix->new([8],[7]),
                Math::Matrix->new([9],[2]),
                Math::Matrix->new([9],[4]),
                Math::Matrix->new([9],[8]),
                Math::Matrix->new([10],[1]),
                Math::Matrix->new([10],[3]),
                Math::Matrix->new([10],[7]),
                Math::Matrix->new([10],[9]),
               );
  my $u_m2 = U_2x2()*M2_2x2();
  my %seen;
  foreach my $v (@v_list) {
    $seen{T_2x2()*$v} = $v;
  }
  foreach my $v (@v_list) {
    # my $key = (U_2x2()*m2p*$v).'';
    my $key = (D_2x2()*$v).'';
    if ($seen{$key}) {
      print "seen key\n$key";
      print "from U*...\n",$v;
      print "and X2*\n",$seen{$key};
      print "\n";
    }
  }
}

{
  # CW filtered descent

  require Math::PlanePath::RationalsTree;
  my $path = Math::PlanePath::RationalsTree->new(tree_type => 'CW');
  my $filtered_xy_is_good = sub {
    my ($x,$y) = @_;
    return (($x + $y) % 2 == 1);
  };
  my $filtered_n_is_good = sub {
    my ($n) = @_;
    my ($x,$y) = $path->n_to_xy($n) or return 0;
    return $filtered_xy_is_good->($x,$y);
  };
  my $filtered_n_children = sub {
    my ($n) = @_;
    my ($x,$y) = $path->n_to_xy($n);
    unless ($filtered_xy_is_good->($x,$y)) {
      die "oops, n=$n xy=$x,$y both odd";
    }
    my @ret;
    foreach my $n_child ($path->tree_n_children($n)) {
      my ($x,$y) = $path->n_to_xy($n_child);
      if ($filtered_n_is_good->($n_child)) {
        push @ret, $n_child;
      } else {
        push @ret, $path->tree_n_children($n_child);
      }
    }
    return @ret;
  };
  #                x,y
  #          -----/   \-----
  #      x,x+y             x+y,y
  #      /   \              /    \
  # x,2x+y  2x+y,x+y  x+y,x+2y  x+2y,y
  #
  # if x odd,y even      x,x+y both odd
  #      __________x,y
  #     /         /   \
  # x,2x+y  2x+y,x+y   x+y,y
  # e,o        e,o       o,e
  # 1 0        2 1       1 1
  # 2 1        1 1       0 1
  # Aup
  #
  # if x even,y odd       x+y,y both odd
  #      x,y__________
  #     /   \         \
  # x,x+y   x+y,x+2y  x+2y,y
  # e,o       o,e        e,o
  # 1 0       1 1        1 2
  # 1 1       1 2        0 1
  #                       D
  my $filtered_xy_children = sub {
    my ($x,$y) = @_;
    if ($x % 2) {   # x odd
      return ($x,2*$x+$y,
              2*$x+$y, $x+$y,
              $x+$y, $y);
    } else {        # x even
      return ($x,$x+$y,
              $x+$y, $x+2*$y,
              $x+2*$y, $y);
    }
  };

  my $count = 0;
  my @pending = ($path->xy_to_n(1,2), $path->xy_to_n(2,1));
  foreach (1 .. 2) {
    my @new_pending;
    while (@pending) {
      my $n = shift @pending;
      $count++;
      my ($x,$y) = $path->n_to_xy($n);
      my @n_children = $filtered_n_children->($n);
      my @xy_children = $filtered_xy_children->($x,$y);
      print "$n  $x,$y  children ",scalar(@n_children),"\n";
      foreach my $n_child (@n_children) {
        push @new_pending, $n_child;
        my ($x_child,$y_child) = $path->n_to_xy($n_child);
        print "   child $n_child  $x_child,$y_child matrix $xy_children[0],$xy_children[1]\n";
        unless ($x_child == $xy_children[0]
                && $y_child == $xy_children[1]) {
          warn "diff $x_child,$y_child matrix $xy_children[0],$xy_children[1]";
        }
        shift @xy_children;
        shift @xy_children;
      }
    }
    @pending = @new_pending;
  }
  print "count $count\n";
  exit 0;
}

{
  # SB legs
  foreach my $elem ([2,1,  3,2, 5,2, 4,1],
                    [3,2,  4,3, 8,5, 7,4],
                    [4,3,  5,4, 11,8, 10,7],
                   ) {
    my ($p,$q, $p1,$q1, $p2,$q2, $p3,$q3) = @$elem;
    {
      my $pp = 2*$p-$q;  # U
      my $qq = $p;
    }
    {
      my $pp = 2*$p-$q;  # U
      my $qq = $p;
    }
  }
  exit 0;
}



{
  # qtree diagram -- unused

  foreach my $tree_type ('UAD','FB','UMT') {
    my $str = <<'HERE';

\Tree [.{001}
        [.{002}
          [{005}
           {006}
           {007} ] ]
        [.{003}
          [{008}
           {009}
           {010} ] ]
        [.{004}
          [{011}
           {012}
           {013} ] ] ]
HERE
    my $path_ab = Math::PlanePath::PythagoreanTree->new(tree_type => $tree_type,
                                                        coordinates => 'AB');
    my $path_pq = Math::PlanePath::PythagoreanTree->new(tree_type => $tree_type,
                                                        coordinates => 'PQ');
    $str =~ s{(\d+)}
             {
               my $field = $1;
               my $len = length($field);
               my ($p,$q) = $path_pq->n_to_xy($field);
               my ($a,$b) = $path_ab->n_to_xy($field);
               my $c = sqrt($a*$a+$b*$b);
               my $str = "$a,$b,$c \\\\ $p,$q";
               sprintf '%-*s', $len, $str
             }ge;
    print $str;
  }
  exit 0;
}

#------------------------------------------------------------------------------
no Smart::Comments;
BEGIN {
  package MyTie::PrintWrap;
  use strict;
  use Text::Tabs;
  use base 'Tie::StdHandle';

  sub PRINT {
    my $self = shift;
    my $str = join('',@_);
    $self->{'column'} ||= 0;
    {
      my $pos = index($str,"\n");
      if ($pos == -1) { $pos = length($str); }
      if ($pos + $self->{'column'} > 79) {
        print "\n";
        $self->{'column'} = 0;
      }
    }
    print $str;
    $self->{'column'} += length($str);
    {
      my $pos = rindex($str,"\n");
      if ($pos >= 0) {
        $self->{'column'} = length(Text::Tabs::expand(substr($str, $pos+1)));
      }
    }
  }
}

# sub path_n_is_row_end {
#   my ($path, $n) = @_;
#   my $depth = $path->tree_n_to_depth($n) // return 0;
#   return ($n == $path->tree_depth_to_n_end($depth));
# }
