#!/usr/bin/perl -w

# Copyright 2013, 2014, 2015, 2017, 2019, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  See the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.


#------------------------------------------------------------------------------
# check MY-BEGIN-PQ-POINTS

{
  while ($triples_tex_str
         =~ /MY-BEGIN-PQ-POINTS(.*?)MY-END-PQ-POINTS/sg) {
    my $content = $1;
    my $linenum = pos_linenum($triples_tex_str);

    my (@p,@q);
    my %seen_pq;
    while ($content =~ m{\((\d+), *(\d+)\)}g) {
      my $p = $1;
      my $q = $2;
      pq_acceptable($p,$q)
        or die "triples.tex:$linenum: MY-BEGIN-PQ-POINTS bad point $p,$q\n";
      push @p, $p;
      push @q, $q;
      if ($seen_pq{"$p,$q"}) {
        die "triples.tex:$linenum: MY-BEGIN-PQ-POINTS duplicate $p,$q\n";
      }
      $seen_pq{"$p,$q"} = 1;
    }

    my $max_p = max(@p);
    foreach my $p (2 .. $max_p) {
      foreach my $q (1 .. $p-1) {
        next unless pq_acceptable($p,$q);
        $seen_pq{"$p,$q"}
          or die "triples.tex:$linenum: MY-BEGIN-PQ-POINTS missing point $p,$q\n";
      }
    }
    print "MY-BEGIN-PQ-POINTS ok ",scalar(@p)," points, line $linenum\n";
  }
}

#------------------------------------------------------------------------------
# check MY-BEGIN-AB-POINTS

{
  while ($triples_tex_str
         =~ /MY-BEGIN-AB-POINTS(.*?)MY-END-AB-POINTS/sg) {
    my $content = $1;
    my $linenum = pos_linenum($triples_tex_str);

    my (@A,@B);
    my %seen_AB;
    while ($content =~ m{\((\d+), *(\d+)\)}g) {
      my $A = $1;
      my $B = $2;
      AB_acceptable($A,$B)
        or die "triples.tex:$linenum: MY-BEGIN-AB-POINTS bad point $A,$B\n";
      push @A, $A;
      push @B, $B;
      if ($seen_AB{"$A,$B"}) {
        die "triples.tex:$linenum: MY-BEGIN-AB-POINTS duplicate $A,$B\n";
      }
      $seen_AB{"$A,$B"} = 1;
    }

    my $max_A = max(@A);
    my $max_B = max(@B);
    foreach my $A (1 .. $max_A) {
      foreach my $B (1 .. $max_B) {
        next unless AB_acceptable($A,$B);
        $seen_AB{"$A,$B"}
          or die "triples.tex:$linenum: MY-BEGIN-AB-POINTS missing point $A,$B\n";
      }
    }
    print "MY-BEGIN-AB-POINTS ok ",scalar(@A)," points, line $linenum\n";
  }

  sub pos_linenum {
    my ($str) = @_;
    $str = substr($str, 0, pos($_[0]));
    return 1 + scalar($str =~ tr/\n//);
  }
}

#------------------------------------------------------------------------------
# check MY-BEGIN-UArD-POINTS

{
  while ($triples_tex_str
         =~ /MY-BEGIN-UArD-ROW-LINES(.*?)MY-END-UArD-ROW-LINES/sg) {
    my $content = $1;
    my $linenum = pos_linenum($triples_tex_str);

    my ($prev_x2,$prev_y2);
    my $count = 0;
    my $next_must_diff_y = 0;
    my $next_must_diff_d = 0;
    while ($content =~ m{\((\d+),*(\d+)\)--\((\d+),*(\d+)\)}g) {
      my $x1 = $1;
      my $y1 = $2;
      my $x2 = $3;
      my $y2 = $4;
      $count++;

      $x2 >= $x1 or die;
      my $length = $x2 - $x1;
      my $d = $x1 - $y1;

      if (line_is_horizontal($x1,$y1, $x2,$y2)) {
        if ($next_must_diff_y) {
          if ($y2 == $prev_y2) {
            die "triples.tex:$linenum: MY-BEGIN-UArD-ROW-LINES at $x1,$y1--$x2,$y2 wanted different from prev_y2=$y2\n";
          }
        }

        my $want_length = 2*$y1 - 2;
        if ($length < $want_length) { # truncated at end of row
          $next_must_diff_y = 1;
        } elsif ($length == $want_length) {
          $next_must_diff_y = 0;
        } else {
          die;
        }

      } elsif (line_is_diagonal($x1,$y1, $x2,$y2)) {
        if ($next_must_diff_d) {
          my $prev_d = $prev_x2 - $prev_y2;
          if ($d == $prev_d) {
            die;
          }
        }

        my $want_length = $d - 2;
        if ($length < $want_length) {
          $next_must_diff_d = 1;
        } elsif ($length == $want_length) {
          $next_must_diff_d = 0;
        } else {
          die;
        }

      } else {
        die "triples.tex:$linenum: MY-BEGIN-UArD-ROW-LINES not horiz or diag\n";
      }

      $prev_x2 = $x2;
      $prev_y2 = $y2;
    }
    print "MY-BEGIN-UArD-ROW-LINES ok $count points, line $linenum\n";
  }
}

#------------------------------------------------------------------------------
# check \label inside align*

{
  my $str = $triples_tex_str;
  $str =~ s/%.*//mg;
  my $count = 0;
  my $bad = 0;
  while ($str =~ m{\\begin\{(align|equation)\*\}
                   (.*?)
                   \\end\{\1\*\}
                  }xsg) {
    my $type = $1;
    my $within = $2;
    $count++;
    if ($within =~ /(\\label(\{.*?\})?)/) {
      print "$type $1\n";
    }
  }
  die if $bad;
  print "$count unlabelled equations\n";
}

#------------------------------------------------------------------------------
